Package: plotdap
Title: Easily Visualize Data from 'ERDDAP™' Servers via the 'rerddap'
        Package
Version: 1.2.0
Date: 2026-05-04
Authors@R: c(
    person("Carson", "Sievert", role = "aut"),
    person("Roy", "Mendelssohn", role = c("aut", "ctb", "cre"), email = "roy.mendelssohn@noaa.gov"))
Description: Easily visualize and animate 'tabledap' and 'griddap' objects obtained via the 'rerddap' package in a simple one-line command,  using either base graphics or 'ggplot2' graphics. 'plotdap' handles extracting and reshaping the data,  map projections and continental outlines.  Optionally the data can be animated through time using the 'gganmiate' package.
License: MIT + file LICENSE
URL: https://github.com/rmendels/plotdap
BugReports: https://github.com/rmendels/plotdap/issues
Depends: R (>= 4.3.0)
Imports: cmocean, dplyr, gganimate, ggnewscale, ggplot2 (>= 3.1.0),
        isoband, lazyeval, lubridate, magrittr, mapdata, maps, raster,
        rerddap (>= 1.0.0), rlang, scales, sf, tidyr, viridis
Suggests: Cairo, knitr, plotly, rmarkdown, testthat
LazyData: true
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-08 16:12:18 UTC; rmendels
Author: Carson Sievert [aut],
  Roy Mendelssohn [aut, ctb, cre]
Maintainer: Roy Mendelssohn <roy.mendelssohn@noaa.gov>
Repository: CRAN
Date/Publication: 2026-05-09 14:41:33 UTC
Built: R 4.5.2; ; 2026-05-09 17:09:43 UTC; unix

Package: plotdap
Title: Easily Visualize Data from 'ERDDAP™' Servers via the 'rerddap'
        Package
Version: 1.2.0
Date: 2026-05-04
Authors@R: c(
    person("Carson", "Sievert", role = "aut"),
    person("Roy", "Mendelssohn", role = c("aut", "ctb", "cre"), email = "roy.mendelssohn@noaa.gov"))
Description: Easily visualize and animate 'tabledap' and 'griddap' objects obtained via the 'rerddap' package in a simple one-line command,  using either base graphics or 'ggplot2' graphics. 'plotdap' handles extracting and reshaping the data,  map projections and continental outlines.  Optionally the data can be animated through time using the 'gganmiate' package.
License: MIT + file LICENSE
URL: https://github.com/rmendels/plotdap
BugReports: https://github.com/rmendels/plotdap/issues
Depends: R (>= 4.3.0)
Imports: cmocean, dplyr, gganimate, ggnewscale, ggplot2 (>= 3.1.0),
        isoband, lazyeval, lubridate, magrittr, mapdata, maps, raster,
        rerddap (>= 1.0.0), rlang, scales, sf, tidyr, viridis
Suggests: Cairo, knitr, plotly, rmarkdown, testthat
LazyData: true
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-08 16:12:18 UTC; rmendels
Author: Carson Sievert [aut],
  Roy Mendelssohn [aut, ctb, cre]
Maintainer: Roy Mendelssohn <roy.mendelssohn@noaa.gov>
Repository: CRAN
Date/Publication: 2026-05-09 14:41:33 UTC
Built: R 4.6.0; ; 2026-05-09 18:56:15 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,ggplotdap)
S3method(print,plotdap)
export("%>%")
export(add_ggplot)
export(add_griddap)
export(add_griddap_contours)
export(add_tabledap)
export(bbox_set)
export(plotdap)
import(mapdata)
import(rerddap)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,theme_bw)
importFrom(magrittr,"%>%")
importFrom(maps,map)
importFrom(raster,brick)
importFrom(raster,calc)
importFrom(raster,crs)
importFrom(raster,extent)
importFrom(raster,isLonLat)
importFrom(raster,ncell)
importFrom(raster,ncol)
importFrom(raster,nlayers)
importFrom(raster,nrow)
importFrom(raster,plot)
importFrom(raster,projectRaster)
importFrom(raster,raster)
importFrom(raster,rasterToPolygons)
importFrom(raster,resample)
importFrom(raster,setExtent)
importFrom(raster,values)
importFrom(rlang,.data)
importFrom(scales,col_numeric)
importFrom(scales,rescale)
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_crs)
importFrom(sf,st_graticule)
importFrom(sf,st_transform)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(tidyr,gather_)
importFrom(utils,getFromNamespace)
importFrom(utils,packageVersion)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# plotdap

The `plotdap` package makes it easy to map data acquired via the
`rerddap` functions `tabledap()` or `griddap()`.

## Installation

You can install the released version of plotdap from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("plotdap")
```

The development version is available from Github and can be installed
with:

``` r
devtools::install_github('rmendels/plotdap')
```

## Functions

**plotdap** has six main functions and can produce plots in both base
graphics and `ggplot2` graphics, though certain features ony work wih
`ggplot2`:

- `plotdap()` which sets up the base map:

  - `plotdap(method = c("ggplot2", "base"), mapData = maps::map("world",        plot = FALSE, fill = TRUE), crs = NULL, datum = sf::st_crs(4326),        mapTitle = NULL, mapFill = "gray80", mapColor = "gray90", ...)`

- `add_tabledap()` which adds the results from a `rerddap::tabledap()`
  call:

  - `add_tabledap(plot, table, var, color = c("#132B43", "#56B1F7"),              size = 1.5, shape = 19, animate = FALSE,              cumulative = FALSE, ...)`

- `add_griddap()` which adds the results from a `rerddap::griddap()`
  call:

  - `add_griddap(plot, grid, var, fill = "viridis", maxpixels = 10000,            time = mean, animate = FALSE, cumulative = FALSE, ...)`

- `add_ggplot()` allows customization of the plot when `ggplot2` is
  used:

  - `add_ggplot(plot, ...)`

- `bbox_set()` which resets the bounding box as well as prepare the
  object foe interactive plotting:

  - ‘bbox_set(plotobj, landmask = TRUE, xlim = NULL, ylim = NULL,
    interactive = FALSE)’

- `add_griddap_contours()` which adds contours to a `add_griddap()`
  object:

  - `add_griddap_contours <- function(p,                            data,                            var,                            n_breaks  = 10L,                            breaks    = NULL,                            lon_col   = "longitude",                            lat_col   = "latitude",                            color     = "black",                            linewidth = 0.3,                            alpha     = 1,                            crs       = 4326L)`

## Examples

Extensive examples are gven in the vignette, which can also be viewed
at:

<https://rmendels.github.io/Using_plotdap.html>

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include = FALSE----------------------------------------------------------
### needed libraries
library(mapdata)
library(plotdap)
library(rerddap)


## ----eval = FALSE-------------------------------------------------------------
# install.packages("plotdap")

## ----eval = FALSE-------------------------------------------------------------
# devtools::install_github('rmendels/plotdap')

## ----world, fig.align = 'center', fig.height = 4, fig.width = 5---------------
library(plotdap)
plotdap()
plotdap("base")

## ----southPole, fig.align = 'center', fig.height = 4, fig.width = 5-----------
plotdap("base",
  mapTitle = "MODIS South Pole Stereographic", 
  mapFill = "transparent", 
  mapColor = "steelblue",
  crs = "+proj=stere +lat_0=-90 +lat_ts=-90 +lon_0=-63 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
)

## ----alaska, fig.align = 'center', fig.height = 4, fig.width = 5--------------
alaska <- "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
plotdap("base", crs = alaska)

## ----echo = FALSE-------------------------------------------------------------
alaska <- "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"

## ----usmap, fig.align = 'center', fig.height = 4, fig.width = 5---------------
library(sf)
library(mapdata)
w <- st_as_sf(maps::map("world", plot = FALSE, fill = TRUE))
us <- st_transform(subset(w, ID == "USA"), alaska)
plotdap(mapData = us)

## ----get_viirsSST, eval = FALSE-----------------------------------------------
# sstInfo <- rerddap::info('erdVHsstaWS3day', url = 'https://coastwatch.pfeg.noaa.gov/erddap/')
# viirsSST <- rerddap::griddap(sstInfo,
#                              latitude = c(41., 31.),
#                              longitude = c(-128., -115),
#                              time = c('2026-03-18T12:00:00Z','2026-03-18T12:00:00Z'),
#                              fields = 'sst')
# 

## ----viirs_hires,echo = TRUE, eval = FALSE------------------------------------
# w <- map("worldHires", xlim = c(-140., -114), ylim = c(30., 42.),
#          fill = TRUE, plot = FALSE)
# # map using that outline,  temperature color from cmocean
# add_griddap(plotdap(mapData = w), viirsSST, ~sst, fill = "thermal" )
# 

## ----world2Hires, fig.align = 'center', fig.height = 3, fig.width = 5,  message = FALSE, warning = FALSE----
xpos <- c(135.25, 240.25)
ypos <- c(20.25, 60.25)
zpos <- c(70.02, 70.02)
remove <- c("UK:Great Britain", "France", "Spain", "Algeria", "Mali", "
            Burkina Faso", "Ghana", "Togo")
#subset world2Hires with those countries removed
w <- map("world2Hires", plot = FALSE, fill = TRUE, ylim = ypos, xlim = xpos)
w <- map("world2Hires", regions = w$names[!(w$names %in% remove)], 
         plot = FALSE, fill = TRUE, ylim = ypos, xlim = xpos)
# plot result
plotdap(mapData = w)


## ----cairo, eval = FALSE,  echo = TRUE, warning = FALSE, message = FALSE------
# # write plot to disk using the Cairo package
# library(Cairo)
# # (latitude limits) / (longitude limits)
# r <- 85 / 120
# CairoPNG("myPlot.png", height = 400 * r, width = 400, res = 96)
# # alter default margins for base plotting
# # (leaving just enough space for a title)
# par(mar = c(0, 0, 1, 0))
# plotdap("base", mapData = us, mapTitle = "Albers projection of Alaska")
# dev.off()

## ----get_sardines, echo = TRUE, eval = FALSE----------------------------------
# my_url <- 'https://coastwatch.pfeg.noaa.gov/erddap/'
# sardines <- tabledap(
#   'FRDCPSTrawlLHHaulCatch',
#   fields = c('latitude',  'longitude', 'time', 'scientific_name',
#              'subsample_count'),
#   'time>=2010-01-01', 'time<=2012-01-01', 'scientific_name="Sardinops sagax"',
#    url = my_url)

## ----echo = TRUE, eval = FALSE------------------------------------------------
# p1 <- add_tabledap(
#   plotdap(crs = "+proj=robin",  mapTitle = "subsample count"),
#   sardines,
#   ~subsample_count
# )
# p2 <- add_tabledap(
#   plotdap(crs = "+proj=robin", mapTitle = "Log subsample count"),
#   sardines,
#   ~log2(subsample_count)
# )
# 
# p1
# p2
# 

## ----echo = TRUE, eval = FALSE------------------------------------------------
# p1 <- add_tabledap(
#   plotdap(crs = "+proj=robin", mapTitle = "Sardines - change color"),
#   sardines,
#   ~subsample_count,
#   color = "dense",
# )
# p2  <- add_tabledap(
#   plotdap(crs = "+proj=robin", mapTitle = "Sardines - change shape and size"),
#   sardines,
#   ~subsample_count,
#   shape = 4,
#   size = 1.
# )
# p1
# p2

## ----get_mur, eval = FALSE----------------------------------------------------
# murSST_west <- griddap(
#   'jplMURSST41',
#   latitude = c(22, 51),
#   longitude = c(-140, -105),
#   time = c('last', 'last'),
#   fields = 'analysed_sst'
#   )

## ----plot_mur, echo = TRUE, eval = FALSE--------------------------------------
# add_griddap(
#   plotdap(crs = "+proj=robin"),
#   murSST_west,
#   ~analysed_sst,
#   maxpixels = 50000
# )

## ----get_wind, eval = FALSE---------------------------------------------------
# wind <- griddap(
#   'erdQMwindmday',
#   time = c('2016-04-16', '2016-06-16'),
#   latitude = c(30, 50),
#   longitude = c(210, 240),
#   fields = 'y_wind'
# )

## ----echo = TRUE, eval = FALSE------------------------------------------------
# p1 <- add_griddap(
#   plotdap(mapTitle = "Mean Meridional Wind"),
#   wind,
#   ~y_wind,
#   fill = "delta",
#   time = mean
# )
# my_func <- function(x) var(x, na.rm = TRUE)
# p2 <- add_griddap(
#   plotdap(mapTitle = "Variance of Meridional Wind"),
#   wind,
#   ~y_wind,
#   fill = "delta",
#   time = my_func
# )
# p1
# p2
# 

## ----viirsSST_gridland, echo = TRUE, eval = FALSE-----------------------------
# plotdap(mapTitle = "Grid over Land") %>%
#       add_griddap(
#         viirsSST,
#         ~sst,
#         fill = "thermal"
#         )
# 

## ----viirsSST_landgrid, echo = TRUE, eval = FALSE-----------------------------
# plotdap(mapTitle = "Land Over Grid") %>%
#     add_griddap(
#       viirsSST,
#       ~sst,
#       fill = "thermal"
#       ) %>%
#     print(landmask = TRUE)

## ----maxpix_10, echo = TRUE, eval = FALSE-------------------------------------
# plotdap(mapTitle = "maxpixels = 10,000") %>%
#     add_griddap(
#       viirsSST,
#       ~sst,
#       fill = "thermal",
#       maxpixels = 10000
#       )

## ----maxpix_50, echo = TRUE, eval = FALSE-------------------------------------
# plotdap(mapTitle = "maxpixels = 50,000") %>%
#      add_griddap(
#        viirsSST,
#        ~sst,
#        fill = "thermal",
#        maxpixels = 50000
#        )

## ----maxpix_100, echo = TRUE, eval = FALSE------------------------------------
# plotdap(mapTitle = "maxpixels = 100,000") %>%
#     add_griddap(
#       viirsSST,
#       ~sst,
#       fill = "thermal",
#       maxpixels = 100000
#       )

## ----maxpix_100_mask, echo = TRUE, eval = FALSE-------------------------------
# plotdap(mapTitle = "maxpixels = 100,000, landmask") %>%
#     add_griddap(
#       viirsSST,
#       ~sst,
#       fill = "thermal",
#       maxpixels = 100000
#       ) %>%
#     print(landmask = TRUE)

## ----hires--------------------------------------------------------------------
w <- map("worldHires", xlim = c(-130., -114), ylim = c(30., 42.), 
         fill = TRUE, plot = FALSE)

## ----viirs_hires_mask, echo = TRUE, eval = FALSE------------------------------
# plotdap(mapData = w) %>%
#     add_griddap(
#       viirsSST,
#       ~sst,
#       fill = 'thermal',
#       maxpixels = 50000
#       ) %>%
#     print(landmask = TRUE)

## ----soda70_get, echo = TRUE, eval = FALSE------------------------------------
# 
# soda70Info <- rerddap::info('erdSoda331oceanmday')
# xpos <- c(135.25, 240.25)
# ypos <- c(20.25, 60.25)
# zpos <- c(70.02, 70.02)
# tpos <- c('2010-12-15', '2010-12-15')
# soda70 <- rerddap::griddap(soda70Info,
#                   longitude = xpos,
#                   latitude = ypos,
#                   time = tpos,
#                   depth = zpos,
#                   fields = 'temp' )
# 

## ----soda70, echo= TRUE, eval = FALSE-----------------------------------------
# remove <- c("UK:Great Britain", "France", "Spain", "Algeria", "Mali",
#             "Burkina Faso", "Ghana", "Togo")
# #subset world2Hires with those countries removed
# w <- map("mapdata::world2Hires", plot = FALSE, fill = TRUE,
#          ylim = ypos, xlim = xpos)
# w <- map("mapdata::world2Hires", regions = w$names[!(w$names %in% remove)],
#          plot = FALSE, fill = TRUE, ylim = ypos, xlim = xpos)
# cols <- cmocean::cmocean('thermal')(256)
# r <- terra::rast(soda70$data[, c("longitude", "latitude", "temp")], type = "xyz")
# r_poly <- terra::as.polygons(r)
# r_sf <- sf::st_as_sf(r_poly)
# sf::st_crs(r_sf) <- 4326
# p <- plotdap(mapData = w)
# p <- add_ggplot(p, ggplot2::geom_sf(data = r_sf, ggplot2::aes(fill = temp), color = NA) )
# p <- add_ggplot(p, ggplot2::scale_fill_gradientn(name ='temp', colors = cols) )
# p
# 

## ----overlay, echo = TRUE, eval = FALSE---------------------------------------
# p <-   add_griddap(plotdap(),
#     murSST_west,
#     ~analysed_sst,
#     fill = "thermal"
#     )
# p1 <-   add_tabledap(p,
#     sardines,
#     ~subsample_count
#     )
# print(p1, landmask = TRUE)

## ----modify, fig.align = 'center', fig.height = 4, fig.width = 5--------------
library(ggplot2)
plotdap(crs = "+proj=robin") %>%
  add_tabledap(
    sardines, 
    ~subsample_count, 
    size = 1
    ) %>%
  add_ggplot(
    labs(
      subtitle = "Sardinops sagax samples",
      caption = "Sardines are yummy"
    ), 
    theme_minimal(),
    theme(axis.ticks = element_blank(), axis.text = element_blank())
  )

## ----modify2, echo = TRUE, eval = FALSE, timeit = TRUE------------------------
# temp_color <- rerddap::colors$temperature
# plotdap(mapTitle = "Reset colorscale limits") %>%
#     add_griddap(
#       viirsSST,
#       ~sst,
#       fill = "thermal"
#       ) %>%
#     add_ggplot(
#        scale_fill_gradientn(colours = temp_color, na.value = NA, limits = c(10, 20)),
#         scale_colour_gradientn(colors = temp_color, na.value = NA, limits = c(10, 20)),
#         guides(colour = 'none')
#      ) %>%
#      print(landmask = TRUE)

## ----eval = FALSE, echo = TRUE------------------------------------------------
# add_tabledap(
#   plotdap(crs = "+proj=robin"),
#   sardines,
#   ~subsample_count,
#   color = "dense",
#   shape = 4,
#   animate = TRUE
# )

## ----eval = FALSE, echo = TRUE------------------------------------------------
# add_griddap(
#   plotdap(crs = "+proj=robin"),
#   wind, ~y_wind,
#   time = identity,
#   fill = 'delta',
#   animate = TRUE
# )

## ----eval = FALSE, echo = TRUE------------------------------------------------
# p <- add_griddap(
#   plotdap(crs = "+proj=robin"),
#   wind, ~y_wind,
#   time = identity,
#   fill = 'delta',
#   animate = TRUE
# )
#   ylim <- c(30, 50)
#   xlim <- c(-150, -120)
#   p <- bbox_set(p, xlim, ylim)

## ----add_contour, eval = FALSE------------------------------------------------
# add_griddap_contours <- function(p,
#                                  data,
#                                  var,
#                                  n_breaks  = 10L,
#                                  breaks    = NULL,
#                                  lon_col   = "longitude",
#                                  lat_col   = "latitude",
#                                  color     = "black",
#                                  linewidth = 0.3,
#                                  alpha     = 1,
#                                  crs       = 4326L)

## ----mur2, eval = FALSE-------------------------------------------------------
# myURL <- "https://coastwatch.pfeg.noaa.gov/erddap/"
# info <- rerddap::info("jplMURSST41", url = myURL)
# murSST  <- griddap(info,
#                    latitude  = c(30, 50),
#                    longitude = c(-140, -110),
#                    time      = c("2020-06-15", "2020-06-15"),
#                    fields    = "analysed_sst")
# 
# p <- plotdap() |>
#       add_griddap(murSST, ~analysed_sst)
# 

## ----countour1, eval = FALSE--------------------------------------------------
# p1 <- p |> add_griddap_contours(murSST$data, "analysed_sst") |> add_ggplot()

## ----contour2, eval = FALSE---------------------------------------------------
# print(p1, landmask = TRUE)
# 

## ----contour3, eval = FALSE---------------------------------------------------
# p |> add_griddap_contours(murSST$data, "analysed_sst",
#                               breaks = c(10, 12, 14, 16, 18, 20)) |>
#                                add_ggplot() |>
#                                print(landmask = TRUE)

## ----echo = TRUE, eval = FALSE------------------------------------------------
# 
# bbox_set <- function(plotobj, landmask = TRUE, xlim = NULL, ylim = NULL, interactive = FALSE)
# 

## ----echo = TRUE, eval = FALSE------------------------------------------------
# library(plotly)
# 
# myURL <- "https://coastwatch.pfeg.noaa.gov/erddap/"
# 
# sstInfo <- rerddap::info('erdVHsstaWS3day', url = myURL)
# # get latest 3-day composite sst
# viirsSST <- rerddap::griddap(sstInfo,
#                              latitude = c(41., 31.),
#                              longitude = c(-128., -115),
#                              time = c('2026-03-18T12:00:00Z','2026-03-18T12:00:00Z'),
#                              fields = 'sst')
# 
# w <- map("worldHires", xlim = c(-140., -114), ylim = c(30., 42.), fill = TRUE, plot = FALSE)
# my_plot <- add_griddap(plotdap(mapData = w), viirsSST, ~sst, fill = "thermal" )
# 
# my_plot1 <- bbox_set(my_plot, xlim = c(-128., -115), ylim = c(31, 41.), interactive = TRUE)
# 
# # must have '"tooltip = "text"' to work properly.
# plotly::ggplotly(my_plot1, tooltip = "text")
# 

#' Add ggplot2 elements to a plotdap object
#'
#' \code{add_ggplot} allows for plotdap ggplot maps to be modified by
#' further ggplot2 settings
#' @param plot a plotdap object.
#' @param ... arguments passed along to \code{geom_sf()}
#' (if \code{method='ggplot2'}, otherwise ignored).
#' @return A plotdap object
#' @export
#' @rdname add_ggplot
#' @examples
#'
#' p <- plotdap(
#'   crs = "+proj=laea +y_0=0 +lon_0=155 +lat_0=-90 +ellps=WGS84 +no_defs")
#' p <- add_ggplot(
#'  p,
#'  ggplot2::theme_bw()
#' )
#'
#'
add_ggplot <- function(plot, ...) {
  if (!is_plotdap(plot)) {
    stop(
      "The first argument to `add_ggplot()` must be a `plotdap()` object",
      call. = FALSE
    )
  }
  dots <- list(...)
  for (i in seq_along(dots)) {
    plot$ggplot <- plot$ggplot + dots[[i]]
  }
  plot
}

#' Add rerddap::griddap() data to a plotdap map
#'
#' \code{add_griddap} adds the data from an 'rerddap::griddap() call to
#' a 'plotdap' map
#' @param plot a \link{plotdap} object.
#' @param grid a \link[rerddap]{griddap} object.
#' @param var a formula defining a variable, or function of variables to visualize.
#' @param fill either a character string of length 1 matching a name in the
#' package \code{cmocean} or a vector of color codes.
#' This defines the colorscale used to encode values
#' of \code{var}.
#' @param maxpixels integer > 0. Maximum number of cells to use for the plot.
#' If maxpixels < ncell(x), sampleRegular is used before plotting.
#' If gridded=TRUE maxpixels may be ignored to get a larger sample
#' @param time how to resolve multiple time frames. Choose one of the following:
#' \itemize{
#'   \item A function to apply to each observation at a particular location
#'   (\link{mean} is the default).
#'   \item A character string (of length 1) matching a time value.
#' }
#' @param animate whether to animate over the \code{time} variable (if it exists).
#' Currently only implemented for \code{method='ggplot2'} and requires the
#' gganimate package.
#' @param cumulative - if animation should be cumulative -default FALSE
#' @param ... arguments passed along to \code{geom_sf()}
#' (if \code{method='ggplot2'}, otherwise ignored).
#' @return A plotdap object
#' @export
#' @rdname add_griddap
#' @examples
#'
#' # base plotting tends to be faster,
#' # but is less extensible plotdap("base")
#'
#' # actual datasets in data folder to meet execution timings
#'
#'
#'  # murSST <- rerddap::griddap(
#'  #  ' jplMURSST41', latitude = c(35, 40), longitude = c(-125, -120.5),
#'  #   time = c('last', 'last'), fields = 'analysed_sst'
#'  # )
#'
#'  # QMwind <- rerddap::griddap(
#'  #  'erdQMwindmday', time = c('2016-11-16', '2017-01-16'),
#'  #  latitude = c(30, 50), longitude = c(210, 240),
#'  #  fields = 'x_wind'
#'  #  )
#'
#' p <- plotdap(crs = "+proj=robin")
#' p <- add_griddap(p, murSST, ~analysed_sst)
#'
#'  # p <- plotdap(mapTitle = "Average wind over time")
#'  # p <- add_griddap(p, QMwind, ~x_wind)
#'
#' # p <- plotdap("base", crs = "+proj=robin")
#' # p <- add_griddap(p, murSST, ~analysed_sst)
#'
#' # layer tables on top of grids
#' require(magrittr)
#' p <- plotdap("base") %>%
#'   add_griddap(murSST, ~analysed_sst) %>%
#'   add_tabledap(sardines, ~subsample_count)
#'
#' # multiple time periods
#' p <- plotdap("base", mapTitle = "Average wind over time")
#' p <- add_griddap(p, QMwind, ~x_wind)
#'

add_griddap <- function(plot, grid, var, fill = "viridis",
                        maxpixels = 10000, time = mean, animate = FALSE,
                        cumulative = FALSE, ...) {
  if (!is.grid(grid))
    stop("The `grid` argument must be a `griddap()` object", call. = FALSE)
  if (!lazyeval::is_formula(var))
    stop("The `var` argument must be a formula", call. = FALSE)
  if (!is.function(time) && !is.character(time))
    stop("The `time` argument must be a function or a character string",
         call. = FALSE)

  # create raster object from filename;
  # otherwise create a sensible raster from data
  r <- get_raster(grid, var)

  # checks for naming and numeric lat/lon
  latlon_is_valid(r)
  # adjust to ensure everthing is on standard lat/lon scale
  r <- latlon_adjust(r)

  # if necessary, reduce a RasterBrick to a RasterLayer
  # http://gis.stackexchange.com/questions/82390/summarize-values-from-a-raster-brick-by-latitude-bands-in-r
  if (body(time) == 'UseMethod("mean")') {
    time <- function(x) mean(x, na.rm = TRUE)
  }
  if (raster::nlayers(r) > 1) {
    if (is.function(time)) {
      r <- raster::calc(r, time)
    } else {
      nm <- make.names(time)
      if (!nm %in% names(r)) {
        warning(
          "The `time` argument doesn't match any of time values.\n",
          sprintf(
            "Valid options include: '%s'",
            paste(unique(grid$data$time), collapse = "', '")
          ),
          call. = FALSE
        )
      }
      r <- r[[nm]]
    }

    if (raster::nlayers(r) > 1 && !animate) {
      stop(
        "The `time` argument hasn't reduced the raster down to a single layer.\n",
        "Either set `animate=TRUE` or provide a suitable value to `time`.",
        call. = FALSE
      )
    }
  }

  # simplify raster, if necessary
  n <- raster::ncell(r)
  if (n > maxpixels) {
    message("grid object contains more than ", maxpixels, " pixels")
    message("increase `maxpixels` for a finer resolution")
    rnew <- raster::raster(
      nrow = floor(raster::nrow(r) * sqrt(maxpixels / n)),
      ncol = floor(raster::ncol(r) * sqrt(maxpixels / n)),
      crs = raster::crs(r),
      ext = raster::extent(r)
    )
    if (inherits(r, "RasterBrick")) {
      for (i in seq_len(raster::nlayers(r))) {
        #r[[i]] <- raster::resample(r[[i]], rnew, method = 'bilinear')
        junk <- raster::resample(r[[i]], rnew, method = 'bilinear')
        if (i == 1) {
          temp <- junk
        } else {
          temp <- raster::addLayer(temp, junk)
        }
      }
      r <- raster::brick(temp)
    } else {
      r <- raster::resample(r, rnew, method = 'bilinear')
    }
  }

  # assumes we apply sf::st_crs() to plot on initiation
  if (inherits(plot$crs, "crs")) {
    crs_string <- plot$crs$proj4string
    if (!is.na(plot$crs$epsg)) {
      epsg_string <- paste0("+init=epsg:", plot$crs$epsg)
      crs_string <- paste(epsg_string, crs_string)
    }
    #r <- raster::projectRaster(r, crs = plot$crs$proj4string)
    r <- raster::projectRaster(r, crs = crs_string)
  }

  # color scale
  # cols <- if (length(fill) == 1) rerddap::colors[[fill]] else fill
  if (length(fill) == 1) {
     if (fill == 'viridis') {
       cols <- viridis::viridis(256)
     } else {
       cols <- cmocean::cmocean(fill)(256)

     }
  }  else {
     cols <- fill
  }

  if (is_ggplotdap(plot)) {
    # TODO: not the most efficient approach, but it will have to do for now
    # https://twitter.com/hadleywickham/status/841763265344487424
    s <- sf::st_as_sf(raster::rasterToPolygons(r))
    vars <- setdiff(names(s), "geometry")
    var_name <- lazyeval::f_text(var)
    variable_name <- "variable"
    sg <- sf::st_as_sf(tidyr::gather(s, {{variable_name}},{{var_name}}, vars))
    if (animate) {
      try_gganimate()
      plot$animate <- TRUE
      plot$nper <- length(s) - 1
      plot$ggplot <- plot$ggplot +
        gganimate::transition_manual(variable, cumulative = cumulative) +
        ggplot2::labs(title = "{current_frame}")
    }

    return(
      add_ggplot(
        plot,
        geom_sf(data = sg,
              mapping = ggplot2::aes(fill = .data[[var_name]], colour = .data[[var_name]]), ...),
        #geom_sf(data = sg,
                #mapping = ggplot2::aes_string(fill = var_name, colour = var_name), ...),
        scale_fill_gradientn(name = var_name, colors = cols),
        scale_colour_gradientn(colors = cols)
        #ggplot2::guides(colour = "none")
      )
    )
  }

  if (animate) {
    warning(
      "Animations are currently only implemented for `method='ggplot2'`",
      call. = FALSE
    )
  }

  # TODO: more props!
  grid <- structure(
    r, props = list(
      name = lazyeval::f_text(var),
      values = raster::values(r),
      color = cols
    )
  )

  # Throw a warning if the grid extent overlaps with another grid?
  plot$layers <- c(
    plot$layers, list(grid)
  )

  plot
}

#' Visualize rerddap data
#'
#' Visualize data returned from rerddap servers. Use \code{plotdap()} to initialize
#' a plot, specify the plotting method (specifically, 'base' or 'ggplot2'),
#' and set some global options/parameters. Then use \code{add_tabledap()}
#' and/or \code{add_griddap()} to add "layers" of actual data to be visualized.
#'
#' @details The "ggplot2" method is slower than "base" (especially
#' for high-res grids/rasters), but is more flexible/extensible. Additional ggplot2
#' layers, as well as scale defaults, labels, theming, etc. may be modified via
#' the \code{add_ggplot()} function. See the mapping vignette for an introduction
#' and overview of rerddap's visualization methods --
#' \code{browseVignettes(package = "rerddap")}.
#'
#' @param method the plotting method. Currently ggplot2 and base plotting
#' are supported.
#' @param mapData an object coercable to an sf object via \code{st_as_sf()}.
#' @param crs a coordinate reference system: integer with the epsg code,
#' or character with proj4string.
#' @param datum crs that provides datum to use when generating graticules.
#' Set to \code{NULL} to hide the graticule.
#' @param mapTitle a title for the map.
#' @param mapFill fill used for the map.
#' @param mapColor color used to draw boundaries of the map.
#' @param ... arguments passed along to \code{geom_sf()}
#' (if \code{method='ggplot2'}, otherwise ignored).
#' @return A plotdap object
#' @export
#' @seealso \code{tabledap()}, \code{griddap()}
#' @author Carson Sievert
#' @examples
#'
#' # base plotting tends to be faster (especially for grids),
#' # but is less extensible plotdap("base")
#'
#'  plotdap()
#'  plotdap("base")
#'
#'

plotdap <- function(method = c("ggplot2", "base"),
                    mapData = maps::map('world', plot = FALSE, fill = TRUE),
                    crs = NULL, datum = sf::st_crs(4326),
                    mapTitle = NULL, mapFill = "gray80", mapColor = "gray90",
                    ...) {

  method <- match.arg(method, method)

  # packages used in both methods
  #require_packages(c("sf", "maps", "lazyeval", "rgeos", "rgdal", "maptools"))
  #require_packages(c("sf", "maps", "lazyeval",  "maptools"))

  # maps is bad at namespacing
  try_require("mapdata", "plotdap")
  bgMap <- sf::st_as_sf(mapData)

  # transform background map to target projection
  if (!is.null(crs)) {
    bgMap <- sf::st_transform(bgMap, crs)
  }

  if (identical(method, "ggplot2")) {
    p <- ggplot() +
      geom_sf(data = bgMap, fill = mapFill, color = mapColor, ...) +
      theme_bw() + ggtitle(mapTitle)
    # keep track of some "global" properties...
    # this seems to be the only way to do things like train axis ranges
    # without a ggplot_build() hack...
    p2 <- list(
      ggplot = p,
      crs = sf::st_crs(bgMap),
      datum = datum
    )
    # return(structure(p2, class = c("ggplotdap", class(p))))
    return(structure(p2, class = "ggplotdap"))
  }

  # construct an custom object for base plotting...
  # we might do all the plotting in the print step since we won't know the
  # final x/y limits until then
  dap <- list(
    crs = sf::st_crs(bgMap),
    datum = datum,
    mapTitle = mapTitle,
    mapColor = mapColor,
    mapFill = mapFill,
    layers = list(bgMap)
  )
  structure(dap, class = "plotdap")
}








#' Print a ggplot plotdap object
#'
#' @param x a ggplotdap object
#' @param ... currently unused
#' @export
print.ggplotdap <- function(x, ...) {
    # find a sensible x/y range....assuming all the layer data is sf
  gg <- x$ggplot
  dots <- list(...)
  if (isTRUE(dots$landmask)) {
    gg$layers[1:2] <- rev(gg$layers[1:2])
    layer_data <- lapply(gg$layers, function(y) y$layer_data(gg$data))
    bbs <- lapply(layer_data[1], sf::st_bbox)
    x$ggplot <- gg
  } else {
    layer_data <- lapply(gg$layers, function(y) y$layer_data(gg$data))
    bbs <- lapply(layer_data[-1], sf::st_bbox)
  }
  xlim <- Reduce(range, lapply(bbs, "[", c("xmin", "xmax")))
  ylim <- Reduce(range, lapply(bbs, "[", c("ymin", "ymax")))
  suppressMessages(x <- add_ggplot(
    x, coord_sf(
      crs = x$crs, datum = x$datum,
      xlim = xlim, ylim = ylim
    )
  ))
#  x <- add_ggplot(
#    x, coord_sf(
#      crs = x$crs, datum = x$datum,
#      xlim = xlim, ylim = ylim
#    )
#  )

  if (isTRUE(x$animate)) {
    print(gganimate::animate(x$ggplot, nframes = x$nper,
                             fps = 1, duration = x$nper))
    invisible(x)
  } else {
    print(x$ggplot)
    invisible(x)
  }
}



#' Print a plotdap object
#'
#' @param x a plotdap object
#' @param ... currently unused
#' @export
print.plotdap <- function(x, ...) {

  # remember, unlike ggplotdap, plotdap layers can have both sf and raster objs
  bbs <- lapply(x$layers[-1], get_bbox)
  xlim <- Reduce(range, lapply(bbs, "[", c("xmin", "xmax")))
  ylim <- Reduce(range, lapply(bbs, "[", c("ymin", "ymax")))

  graticule <- sf::st_graticule(
    c(xlim[1], ylim[1], xlim[2], ylim[2]) %||% get_bbox(x$layers[[1]]),
    crs = x$crs,
    datum = x$datum
  )

  # plot the background map
  plot(
    x$layers[[1]],
    xlim = xlim,
    ylim = ylim,
    main = x$mapTitle %||% "",
    col = x$mapFill,
    border = x$mapColor,
    graticule = graticule,
    #setParUsrBB = TRUE,
    bty = "n",
    reset = FALSE,
    ...
  )

  for (i in setdiff(seq_along(x$layers), 1)) {
    layer <- x$layers[[i]]
    props <- attr(layer, "props")
    rng <- range(props$values, na.rm = TRUE)
    pal <- scales::col_numeric(props$color, rng)
    breaks <- stats::quantile(props$values, 0:4/4, na.rm = TRUE)

    # plot rasters first, otherwise we have to fiddle with legends
    if (is_raster(layer)) {
      raster::plot(
        layer,
        add = TRUE,
        legend = FALSE,
        frame.plot = FALSE,
        # TODO: where does this scaling occur? In raster::plot()?
        col = props$color,
        ...
      )
      graphics::legend(
        "bottomright",
        title = props$name,
        legend = format(breaks),
        col = pal(breaks),
        pch = 20
      )
    } else {
      # should be sf POINTS
      plot(
        layer[["geometry"]],
        add = TRUE,
        col = pal(props$values),
        pch = props$shape,
        cex = props$size
      )
      graphics::legend(
        "topright",
        title = props$name,
        legend = format(breaks),
        col = pal(breaks),
        pch = 20
      )
    }
  }
  x
}


# ------------------------------------------------------------------------
# Internal helper functions
# ------------------------------------------------------------------------

is_plotdap <- function(x) {
  inherits(x, c("ggplotdap", "plotdap"))
}

is_ggplotdap <- function(x) {
  is_plotdap(x) && ggplot2::is.ggplot(x$ggplot)
}

is_raster <- function(x) {
  inherits(x, c("RasterLayer", "RasterBrick", "RasterStack"))
}

get_bbox <- function(x) {
  if (inherits(x, "sf")) {
    return(sf::st_bbox(x))
  }
  # TODO: support raster objects, as well?
  if (inherits(x, "RasterLayer")) {
    ext <- as.list(raster::extent(x))
    return(stats::setNames(ext, c("xmin", "xmax", "ymin", "ymax")))
  }
  invisible()
}

get_raster <- function(grid, var) {
  times <- grid$summary$dim$time$vals
  lats <- grid$summary$dim$latitude$vals
  lons <- grid$summary$dim$longitude$vals
  ylim <- range(lats, na.rm = TRUE)
  xlim <- range(lons, na.rm = TRUE)
  ext <- raster::extent(xlim[1], xlim[2], ylim[1], ylim[2])
  r <- if (length(times) > 1) {
    # ensure values appear in the right order...
    # TODO: how to detect a south -> north ordering?
    if ("lat" %in% names(grid$data)) {
      d <- dplyr::arrange(grid$data, time, desc(lat), lon)
    } else {
      d <- dplyr::arrange(grid$data, time, desc(latitude), longitude)
    }
    b <- raster::brick(
      nl = length(times),
      nrows = length(lats),
      ncols = length(lons)
    )
    raster::values(b) <- lazyeval::f_eval(var, d)
    raster::setExtent(b, ext)
  } else {
    if ("lat" %in% names(grid$data)) {
      d <- dplyr::arrange(grid$data, desc(lat), lon)
    } else {
      d <- dplyr::arrange(grid$data, desc(latitude), longitude)
    }
    raster::raster(
      nrows = length(lats),
      ncols = length(lons),
      ext = ext,
      vals = lazyeval::f_eval(var, d)
    )
  }
  names(r) <- make.names(unique(grid$data$time) %||% "")
  r
}


utils::globalVariables(c("time", "desc", "lat", "lon", "latitude", "longitude"))

latlon_is_valid <- function(x) {
  if (is_raster(x)) {
    if (!raster::isLonLat(x)) {
      stop(
        "raster object must have a longitude/latitude coordinate reference system (CRS)",
        call. = FALSE
      )
    }
    return(TRUE)
  }

  if (!is.numeric(latValues(x))) {
    stop("Latitudes must be numeric", call. = FALSE)
  }

  if (!is.numeric(lonValues(x))) {
    stop("Longitudes must be numeric", call. = FALSE)
  }

  invisible(TRUE)
}

latlon_adjust <- function(x) {
  if (is.table(x)) {

    lonIDX <- grep(lonPattern(), names(x))
    lon <- range(x[[lonIDX]], na.rm = TRUE)

    if (all(dplyr::between(lon, 0, 180))) {

      warning(
        "Can't determine whether longitude is on (0, 360) or (-180, 180) scale\n",
        "Defaulting to (-180, 180) scale...",
        call. = FALSE
      )

    } else if (all(dplyr::between(lon, 180, 360))) {

      #idx <- isTRUE(lon > 180)
      x[[lonIDX]] <- lon - 360

    } else if (all(dplyr::between(lon, -180, 180))) {

      # nothing to do....

    } else {

      # TODO: report the invalid values?
      warning("Invalid longitude values", call. = TRUE)

    }

    return(x)
  }

  if (is_raster(x)) {

    ext <- raster::extent(x)
    lon <- c(ext@xmin, ext@xmax)

    if (all(dplyr::between(lon, 0, 180))) {

      warning(
        "Can't determine whether longitude is on (0, 360) or (-180, 180) scale\n",
        "Defaulting to (-180, 180) scale...",
        call. = FALSE
      )

    } else if (all(dplyr::between(lon, 180, 360))) {

      newExt <- raster::extent(
        c(c(ext@xmin, ext@xmax) - 360, ext@ymin, ext@ymax)
      )
      x <- raster::setExtent(x, newExt, keepres = TRUE)

    } else if (all(dplyr::between(lon, -180, 180))) {

      # nothing to do....

    } else if (all(dplyr::between(lon, 0, 360))) {

      # nothing to do....
    } else {

      # TODO: report the invalid values?
      warning("Invalid longitude values", call. = TRUE)

    }

    return(x)
  }

  # throw an error?
  x
}


latValues <- function(x) {
  latIDX <- grep(latPattern(), names(x))
  if (length(latIDX) != 1) {
    stop(
      "Couldn't find latitude variable. Must be named one of the following:\n",
      "'lat', 'lats', 'latitude'",
      call. = FALSE
    )
  }
  x[[latIDX]]
}

lonValues <- function(x) {
  lonIDX <- grep(lonPattern(), names(x))
  if (length(lonIDX) != 1) {
    stop(
      "Couldn't find longitude variable. Must be named one of the following:\n",
      "'lon', 'lons', 'longitude'",
      call. = FALSE
    )
  }
  if (!is.numeric(x[[lonIDX]])) {
    stop("Longitude must be numeric", call. = FALSE)
  }
  x[[lonIDX]]
}

latPattern <- function() "^lat$|^lats$|^latitude$"
lonPattern <- function() "^lon$|^lons$|^longitude$"

# valid names for latitude
latNames <- function() {
  sub("\\$", "", sub("\\^", "", strsplit(latPattern(), "|", fixed = TRUE)[[1]]))
}

lonNames <- function() {
  sub("\\$", "", sub("\\^", "", strsplit(lonPattern(), "|", fixed = TRUE)[[1]]))
}


utils::globalVariables("variable")

#' change bounding box in plotdap object
#'
#' \code{bbox_set}changes the bounding box in an plotdap object.
#' Particularly needed if using gganimate::animate()
#' @export
#' @param plotobj valid plotdap object
#' @param landmask optional: if true land is plotted over the data
#' @param xlim optional: new x-values of the bounding box otherwise taken from the plotdap object
#' @param ylim optional: new y-values of the bounding box otherwise taken from the plotdap object
#' @param interactive optional: if true  creates an interactive version of the graphic
#' @return a ggplot object with reset limits. If interactive is true,  then returns a
#'            ggplot object that will work with the `plotly` package.
#'
#' @examples
#' p <- plotdap()
#' p <- add_tabledap(p, sardines, ~subsample_count)
#' xlim = c(-125, -115)
#' ylim <- c(30., 50.)
#' p <- bbox_set(p, xlim = xlim, ylim = ylim)

bbox_set <- function(plotobj, landmask = TRUE, xlim = NULL, ylim = NULL, interactive = FALSE) {
  if (!("ggplotdap" %in% class(plotobj))) {
    stop("object passed not a plotdap object")
  }
  if (!is.null(xlim)){
    if (!(length(xlim) == 2)) {
      stop("xlim does not contain two items")
    }
    if (xlim[1] > xlim[2]) {
      stop("xlim improperly ordered")
    }
  }
  if (!is.null(ylim)){
    if (!(length(ylim) == 2)) {
      stop("ylim does not contain two items")
    }
    if (ylim[1] > ylim[2]) {
      stop("ylim improperly ordered")
    }
  }
  gg <- plotobj$ggplot
  if (interactive) {plotobj <- add_plotdap_tooltips(plotobj)}
  if (landmask) {
    gg$layers[1:2] <- rev(gg$layers[1:2])
    layer_data <- lapply(gg$layers, function(y) y$layer_data(gg$data))
    bbs <- lapply(layer_data[1], sf::st_bbox)
    plotobj$ggplot <- gg
  } else {
    layer_data <- lapply(gg$layers, function(y) y$layer_data(gg$data))
    bbs <- lapply(layer_data[-1], sf::st_bbox)
  }
  xlim <- Reduce(range, lapply(bbs, "[", c("xmin", "xmax")))
  ylim <- Reduce(range, lapply(bbs, "[", c("ymin", "ymax")))
  plotobj$ggplot <- plotobj$ggplot + ggplot2::coord_sf(
                 crs = plotobj$crs, datum = plotobj$datum,
                  xlim = xlim, ylim = ylim
                 )
  plotobj$ggplot
}


# Add tooltip data to plotdap object (modifies layer 2)
add_plotdap_tooltips <- function(plotdap_obj) {

  # Get the data from layer 2
  plot_data <- plotdap_obj$ggplot$layers[[2]]$data

  # Extract centroid coordinates
  coords <- sf::st_coordinates(sf::st_centroid(plot_data$geometry))
  plot_data$lon <- coords[, "X"]
  plot_data$lat <- coords[, "Y"]

  # Get the variable name
  var_name <- setdiff(names(plot_data), c("variable", "geometry", "lon", "lat"))[1]

  # Create tooltip text
  plot_data$hover_text <- paste0(
    "Lon: ", round(plot_data$lon, 3), "\n",
    "Lat: ", round(plot_data$lat, 3), "\n",
    var_name, ": ", round(plot_data[[var_name]], 3)
  )

  # Update the layer
  plotdap_obj$ggplot$layers[[2]]$data <- plot_data
  plotdap_obj$ggplot$layers[[2]]$mapping$text <- quote(hover_text)

  # Return modified object
  return(plotdap_obj)
}

#' @import rerddap
#' @import mapdata
#' @importFrom utils packageVersion getFromNamespace
#' @importFrom stats quantile setNames
#' @importFrom tidyr gather_
#' @importFrom dplyr arrange between
#' @importFrom maps map
#' @importFrom scales rescale col_numeric
#' @importFrom rlang .data
#' @importFrom sf st_as_sf st_transform st_crs st_bbox st_graticule
#' @importFrom ggplot2  ggplot geom_sf guides guide_colorbar coord_sf theme_bw scale_colour_gradientn scale_fill_gradientn ggtitle labs
#' @importFrom raster raster nlayers calc ncell nrow ncol crs extent resample projectRaster rasterToPolygons values plot brick setExtent isLonLat
NULL

#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`

"%||%" <- function(x, y) {
  if (length(x) > 0) x else y
}

is.grid <- function(x) {
  inherits(x, c("griddap_nc", "griddap_csv"))
}

is.table <- function(x) {
  inherits(x, "tabledap")
}

require_packages <- function(x) {
  for (i in x) {
    if (system.file(package = i) == "") {
      stop(sprintf("Please install package: '%s'", i), call. = FALSE)
    }
  }
  invisible(TRUE)
}

# copied from ggplot2:::try_require...thanks Hadley
try_require <- function(package, fun) {
  if (requireNamespace(package, quietly = TRUE)) {
    library(package, character.only = TRUE)
    return(invisible())
  }
  stop("Package `", package, "` required for `", fun, "`.\n",
       "Please install and try again.", call. = FALSE)
}

try_gganimate <- function() {
  if (system.file(package = "gganimate") != "") {
    if (utils::packageVersion("gganimate") > "1.0.0") {
      return(TRUE)
    }
  }
  stop(
    "This functionality requires a recent version of the gganimate package.\n",
    "Please install via devtools:\n",
    "devtools::install_github('dgrtwo/gganimate')",
    call. = FALSE
  )
  FALSE
}

str_trim <- function(str) {
  gsub("^\\s+|\\s+$", "", str)
}

format_table <- function(table, .info) {
  # format lat/lons
  vars <- names(table)
  latIDX <- grep(latPattern(), vars)
  lonIDX <- grep(lonPattern(), vars)

  if (length(latIDX) == 1)  {
    table[[latIDX]] <- as.numeric(str_trim(table[[latIDX]]))
  } else {
    warning(
      "Couldn't automatically determine latitude column. ",
      sprintf("Please name it one of the following: '%s'",
      paste(latNames(), collapse = "', '")),
      call. = FALSE
    )
  }
  if (length(lonIDX) == 1)  {
    table[[lonIDX]] <- as.numeric(str_trim(table[[lonIDX]]))
  } else {
    warning(
      "Couldn't automatically determine longitude column. ",
      sprintf("Please name it one of the following: '%s'",
      paste(lonNames(), collapse = "', '")),
      call. = FALSE
    )
  }

  # format time
  if ("time" %in% vars) {
    # TODO: will this cover all cases?
    #table[["time"]] <- as.Date(table[["time"]], origin = '1970-01-01', tz = "GMT")
    #table[["time"]] <- lubridate::as_datetime(table[["time"]], origin = '1970-01-01', tz = "GMT")
    table[["time"]] <- lubridate::as_datetime(table[["time"]], tz = "GMT")
  }

  table
}

# sf is not properly importing rgeos.
# I need to import it, but causes note if not
# referenced. Here is a dummy function.
#dummy <- function() rgeos::getScale()
# dummy2 <- function() rgdal::checkCRSArgs("+proj=laea +y_0=0 +lon_0=155 +lat_0=-90 +ellps=WGS84 +no_defs")

# This is likely WRONG, SAD!
# rescale_lims <- function(x, .info = info(attr(x, "datasetid"))) {
#
#   # lat/lon can be on different ranges (e.g. [-180, 180] vs [0, 360])
#   # put them on the usual [-180, 180]/[-90, 90] scales
#   for (i in c("latitude", "longitude")) {
#     default <- if (i == "latitude") c(-90, 90) else c(-180, 180)
#     domain <- tryCatch({
#       d <- .info$alldata[[i]]
#       rng <- d[d$attribute_name == "actual_range", "value"]
#       as.numeric(strtrim(strsplit(rng, ",")[[1]]))
#     }, error = function(e) default)
#
#     if (is.table(x)) {
#       x[[i]] <- scales::rescale(x[[i]], default, domain)
#     } else {
#       # griddap
#       x$summary$dim[[i]]$vals <- scales::rescale(
#         x$summary$dim[[i]]$vals, default, domain
#       )
#       # assumes latitude/longitude are abbreviated to lat/lon in the data frame (I think that is safe?)
#       abbr <- substr(i, 1, 3)
#       x$data[, abbr] <- scales::rescale(
#         x$data[, abbr], c(-180, 180), domain
#       )
#     }
#   }
#   x
# }

#' sardine Data
#'
#' pre-Download of sardine data in `add_tabledap()` example so that example
#' can run within CRAN Time limits
#'
#' obtained using the `rerddap` command
#'   sardines <- tabledap( 'FRDCPSTrawlLHHaulCatch',
#'      fields = c('latitude',  'longitude', 'time', '
#'          scientific_name', 'subsample_count'),
#'         'time>=2010-01-01', 'time<=2012-01-01',
#'         'scientific_name="Sardinops sagax"')
#' )
"sardines"

#' plotdap: Easily Visualize Data from 'ERDDAP™' Servers via the 'rerddap' Package
#'
#'
#' Easily visualize and animate 'tabledap' and 'griddap' objects obtained via the
#' 'rerddap' package in a simple one-line command,  using either base graphics or
#' 'ggplot2' graphics. 'plotdap' handles extracting and reshaping the data,  map
#'  projections and continental outlines.  Optionally the data can be animated
#'  through time using the 'gganmiate' package.#'
#'
#' The package contains four main functions:
#'
#' @section Main Functions:
#' \itemize{
#' \item \code{plotdap} - Visualize data returned from rerddap servers.
#' \item \code{add_griddap} - Add rerddap::griddap() data to a plotdap map
#' \item \code{add_tabledap} - Add rerddap::tabledap data to a plotdap map
#' \item \code{add_ggplot} - Add ggplot2 elements to a plotdap object
#'  }
#'#'
#'  @section Details:
#' Besides the terse help documents,  more  detail in using the
#' functions are given  in the included vignette "using_plotdap". The datasets used
#' in the vignette are included in the "data" directory.

#' @keywords internal
#'@aliases plotdap-package
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' QMwind Data
#'
#' pre-Download of QMwind in `add_griddap()` example so that example
#' can run within CRAN Time limits
#'
#' obtained using the `rerddap` command
#' wind <- griddap('erdQMwindmday',
#'        time = c('2016-11-16', '2017-01-16'), latitude = c(30, 50),
#'        longitude = c(210, 240),
#'        fields = 'x_wind')
#' )
"QMwind"

#' murSST Data
#'
#' pre-Download of murSST in `add_griddap()` example so that example
#' can run within CRAN Time limits
#'
#' obtained using the `rerddap` command
#' murSST <- griddap(
#' 'jplMURSST41', latitude = c(22, 51), longitude = c(-140, -105),
#' time = c('last', 'last'), fields = 'analysed_sst'
#' )
"murSST"

#' Add rerddap::tabledap data to a plotdap map
#'
#' \code{add_tabledap} adds the data from an 'rerddap::tabledap()' call to
#' a 'plotdap' map
#' @param plot a \link[plotdap]{plotdap} object.
#' @param table a \link[rerddap]{tabledap} object.
#' @param var a formula defining a variable, or function of variables to visualize.
#' @param color either a character string of length 1 matching a name in \code{cmocean}
#' or a vector of color codes. This defines the colorscale used to encode values
#' of \code{var}.
#' @param size the size of the symbol.
#' @param shape the shape of the symbol. For valid options, see the 'pch' values
#' section on \link{points}. \code{plot(0:25, 0:25, pch = 0:25)} also gives a
#' quick visual of the majority of possibilities.
#' @param animate whether to animate over the \code{time} variable (if it exists).
#' Currently only implemented for \code{method='ggplot2'} and requires the
#' gganimate package.
#' @param cumulative - if animation should be cumulative -default FALSE
#' @param ... arguments passed along to \code{geom_sf()} (if \code{method='ggplot2'}, otherwise ignored).
#' @return A plotdap object
#' @export
#' @rdname add_tabledap
#' @examples
#'
#' # base plotting tends to be faster,
#' # but is less extensible plotdap("base")
#'
#' # test datasets in data folder to meet execution timings
#' # code given to extract the data
#'
#'\donttest{
#' sardines <- rerddap::tabledap(
#'  'FRDCPSTrawlLHHaulCatch',
#'  fields = c('latitude',  'longitude', 'time', 'scientific_name', 'subsample_count'),
#'   'time>=2010-01-01', 'time<=2012-01-01',
#'   'scientific_name="Sardinops sagax"'
#'   )
#'}
#'
#' p <- plotdap()
#' p1 <- add_tabledap(p, sardines, ~subsample_count)
#' p2 <- add_tabledap(p, sardines, ~log2(subsample_count))
#'
#' # using base R plotting
#' p <- plotdap("base")
#' p <- add_tabledap(p, sardines, ~subsample_count)
#'
#' # robinson projection
#' p <- plotdap(crs = "+proj=robin")
#' p <- add_tabledap(p, sardines, ~subsample_count)
#'
#'
add_tabledap <- function(plot, table, var, color = c("#132B43", "#56B1F7"),
                         size = 1.5, shape = 19, animate = FALSE,
                         cumulative = FALSE, ...) {
  if (!is.table(table))
    stop("The `table` argument must be a `tabledap()` object", call. = FALSE)
  if (!lazyeval::is_formula(var))
    stop("The var argument must be a formula", call. = FALSE)

  table <- format_table(table)

  # checks for naming and numeric lat/lon
  latlon_is_valid(table)
  # adjust to ensure everthing is on standard lat/lon scale
  table <- latlon_adjust(table)

  nms <- names(table)
  # convert to sf
  table <- sf::st_as_sf(
    table, crs = sf::st_crs(4326),
    coords = c(grep(lonPattern(), nms), grep(latPattern(), nms))
  )
  # transform to target projection
  if (inherits(plot$crs, "crs")) {
    table <- sf::st_transform(table, plot$crs)
  }

  # color scale
  # cols <- if (length(color) == 1) rerddap::colors[[color]] else color

  if (length(color) == 1) {
    if (color == 'viridis') {
      cols = viridis::viridis(256)
    } else{
      cols <- cmocean::cmocean(color)(256)
    }
  }  else {
    cols <- color
  }


    if (is_ggplotdap(plot)) {

      if (animate && "time" %in% names(table)) {
        try_gganimate()
        plot$animate <- TRUE
        plot$nper <- length(unique(table$time))
        plot$ggplot <- plot$ggplot +
          gganimate::transition_manual(factor(time), cumulative = cumulative) +
          ggplot2::labs(title = "{current_frame}")
      }
      if (length(plot$ggplot$layers) == 1) {
        return(
          add_ggplot(
            plot,
            geom_sf(data = table, mapping = ggplot2::aes(colour = !!rlang::f_rhs(var)),
                    size = size, pch = shape, ...),
            scale_colour_gradientn(name = lazyeval::f_text(var), colours = cols))
        )
      } else {
        #plot$ggplot <- plot$ggplot + ggnewscale::new_scale_colour() + ggnewscale::new_scale_fill()
        return(
          add_ggplot(
            plot,
            ggnewscale::new_scale_colour(),
            geom_sf(data = table, mapping = ggplot2::aes(colour = !!rlang::f_rhs(var)),
                    size = size, pch = shape, ...),
            scale_colour_gradientn(name = lazyeval::f_text(var), colours = cols))
        )
      }
    }


  if (animate) {
    warning(
      "Animations are currently only implemented for `method='ggplot2'`",
      call. = FALSE
    )
  }

  table <- structure(
    table, props = list(
      name = lazyeval::f_text(var),
      values = lazyeval::f_eval(var, table),
      color = cols,
      size = size,
      shape = shape
    )
  )

  plot$layers <- c(plot$layers, list(table))

  plot
}

#' Add contour lines to a plotdap object
#'
#' Computes isolines from a griddap data frame and appends them as a
#' \code{geom_sf()} layer to the \strong{plotdap object's internal ggplot}.
#' Call this \emph{before} \code{add_ggplot()} or \code{print()}.
#'
#' @param p         a plotdap object (result of \code{plotdap()} piped through
#'                  \code{add_griddap()}).  \strong{Not} the result of
#'                  \code{add_ggplot()}.
#' @param data      data frame  typically \code{dat$data} from a
#'                  \code{griddap()} call.
#' @param var       character; name of the variable column to contour.
#' @param n_breaks  integer; number of evenly-spaced interior contour levels
#'                  (ignored when \code{breaks} is supplied). Default 10.
#' @param breaks    numeric vector of explicit contour levels, or \code{NULL}.
#' @param lon_col   name of longitude column (default \code{"longitude"}).
#' @param lat_col   name of latitude column  (default \code{"latitude"}).
#' @param color     contour line color (default \code{"black"}).
#' @param linewidth contour line width (default \code{0.3}).
#' @param alpha     opacity 01 (default \code{1}).
#' @param crs       integer EPSG code for the data (default \code{4326}).
#'
#' @return the plotdap object with contour lines added to its internal ggplot.
#'   Pipe the result to \code{add_ggplot()} or \code{print()} as usual.
#' @export
#'
#' @examples
#' \dontrun{
#'   library(rerddap)
#'   library(plotdap)
#'
#'   myURL <- "https://coastwatch.pfeg.noaa.gov/erddap/"
#'   info <- rerddap::info("jplMURSST41", url = myURL)
#'   dat  <- griddap(info,
#'             latitude  = c(30, 50),
#'             longitude = c(-140, -110),
#'             time      = c("2020-06-15", "2020-06-15"),
#'             fields    = "analysed_sst")
#'
#'   p <- plotdap() |>
#'          add_griddap(dat, ~analysed_sst)
#'
#'   ## Default: 10 evenly-spaced contour levels, then render
#'   p |> add_griddap_contours(dat$data, "analysed_sst") |> add_ggplot()
#'
#'   ## Explicit break values
#'   p |> add_griddap_contours(dat$data, "analysed_sst",
#'                              breaks = c(10, 12, 14, 16, 18, 20)) |> add_ggplot()
#'
#'   ## More levels, thinner grey lines
#'   p |> add_griddap_contours(dat$data, "analysed_sst",
#'                              n_breaks = 20, color = "grey40", linewidth = 0.2) |>
#'        add_ggplot()
#' }
add_griddap_contours <- function(p,
                                 data,
                                 var,
                                 n_breaks  = 10L,
                                 breaks    = NULL,
                                 lon_col   = "longitude",
                                 lat_col   = "latitude",
                                 color     = "black",
                                 linewidth = 0.3,
                                 alpha     = 1,
                                 crs       = 4326L) {

  if (!("ggplotdap" %in% class(p))) {
    stop("object passed not a plotdap object")
  }

  csf <- .make_contour_sf(data, var, n_breaks, breaks, lon_col, lat_col, crs)

  if (is.null(csf)) {
    warning("add_griddap_contours: no contours generated  ",
            "check that '", var, "' exists and has sufficient range")
    return(p)
  }

  # p is a plotdap object; add the layer to its internal ggplot
  p$ggplot <- p$ggplot + ggplot2::geom_sf(
    data        = csf,
    color       = color,
    linewidth   = linewidth,
    alpha       = alpha,
    inherit.aes = FALSE
  )

  p
}


#  internal: build sf MULTILINESTRING object from griddap data.frame

.make_contour_sf <- function(data, var, n_breaks, breaks, lon_col, lat_col, crs) {

  stopifnot(is.data.frame(data), var %in% names(data),
            lon_col %in% names(data), lat_col %in% names(data))

  data <- data[!is.na(data[[var]]), , drop = FALSE]
  if (nrow(data) == 0L) return(NULL)

  lons <- sort(unique(data[[lon_col]]))
  lats <- sort(unique(data[[lat_col]]))

  #  determine breakpoints
  z_range <- range(data[[var]], na.rm = TRUE)

  if (is.null(breaks)) {
    # n_breaks interior levels, evenly spaced; exclude the endpoints so the
    # outermost contours sit inside the data range rather than on the edge
    all_breaks <- seq(z_range[1], z_range[2], length.out = n_breaks + 2L)
    breaks <- all_breaks[-c(1L, length(all_breaks))]
  }

  breaks <- sort(unique(breaks))

  # Remove breaks outside the data range
  breaks <- breaks[breaks > z_range[1] & breaks < z_range[2]]
  if (length(breaks) == 0L) return(NULL)

  #  build z matrix (rows = lat ascending, cols = lon)
  z_mat <- matrix(NA_real_, nrow = length(lats), ncol = length(lons))
  z_mat[cbind(match(data[[lat_col]], lats),
              match(data[[lon_col]], lons))] <- data[[var]]

  #  generate isolines
  iso <- isoband::isolines(x = lons, y = lats, z = z_mat, levels = breaks)

  #  convert each level to one sf MULTILINESTRING row
  rows <- lapply(seq_along(iso), function(i) {
    ln <- iso[[i]]
    if (length(ln$x) == 0L) return(NULL)

    ids  <- unique(ln$id)
    mats <- lapply(ids, function(id) {
      mask <- ln$id == id
      cbind(ln$x[mask], ln$y[mask])
    })

    sf::st_sf(
      level    = breaks[i],
      geometry = sf::st_sfc(sf::st_multilinestring(mats), crs = crs),
      stringsAsFactors = FALSE
    )
  })

  result <- do.call(rbind, Filter(Negate(is.null), rows))
  if (is.null(result) || nrow(result) == 0L) return(NULL)
  result
}

---
title: "Using plotdap"
author: ""
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3

vignette: >
  %\VignetteIndexEntry{Using plotdap}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, include = FALSE}
### needed libraries
library(mapdata)
library(plotdap)
library(rerddap)

```

# 

The R package **plotdap** makes it easy to visualize 'tabledap' and 'griddap' objects obtained via the [**rerddap**](https://CRAN.R-project.org/package=rerddap) package. Maps can be made using either base or ggplot2 graphics, and the user does not need to know the intricacies of obtaining continental outlines, projections, and combining those with the data to form maps. Animations of the data are also readily obtained. `plotdap` works in a similar fashion with either tables or grids return by `rerddap` further simplifying the mapping process.

A user who desires fine control of their maps should learn how to map the data themselves - some examples are give in the `rerddap` vignette. But `plotdap` provides a simplified workflow of obtaining data using `rerddap` and quickly and simply mapping the data. In what follows we go over how to install `plotdap` and it's basic usage, as well as how to utilize some of the more important options in the package in order to improve the map.

A new feature is that the function `bbox_set()` can now be used to create interactive plots using `plotly::ggplotly()`, see [Interactive `plotdap` objects and `bbox_set()`](#interactive).

A second new feature is the function `add_griddap_contours()` which adds contours to any `plotdap::add_griddap()` object, see [Contour Plots](#contours).

## Installation

**plotdap** can be installed from CRAN with:

```{r, eval = FALSE}
install.packages("plotdap")
```

and the development version can be installed with:

```{r, eval = FALSE}
devtools::install_github('rmendels/plotdap')
```

## Getting started with `plotdap()`

The `plotdap()` function makes it easy to visualize data acquired via `rerddap::tabledap()` or `rerddap::griddap()`. Regardless of the data you want to visualize, you'll always want to start a plot via `plotdap()`, where you may specify some "global" plotting options. Subsequent sections will demonstrate how to add tables/grids via `add_tabledap()`/`add_griddap()`, but for now we'll focus on options provided by `plotdap()`. Most importantly, the first argument decides whether base 'R' graphics or [ggplot2](https://cran.r-project.org/package=ggplot2) graphics should be used for the actual plotting.

```{r world, fig.align = 'center', fig.height = 4, fig.width = 5}
library(plotdap)
plotdap()
plotdap("base")
```

In addition to choosing a plotting method, `plotdap()` is where you can define properties of the background map, including the target projection using a valid coordinate reference system (CRS) definition. Projection is performed using the [PROJ.4 library](https://proj.org/), and [spatialreference.org](https://spatialreference.org) is a great resource for finding PROJ.4 CRS descriptions. Using the search utility, you can for example, [search for "South Pole"](https://spatialreference.org/ref/?search=South+Pole&srtext=Search) and pick from a number of options. Here I've chosen the [South Pole Stereographic](https://spatialreference.org/ref/esri/102021/) option and copy-pasted the [Proj4 page](https://spatialreference.org/ref/esri/102021/proj4.txt) with the CRS definition:

```{r southPole, fig.align = 'center', fig.height = 4, fig.width = 5}
plotdap("base",
  mapTitle = "MODIS South Pole Stereographic", 
  mapFill = "transparent", 
  mapColor = "steelblue",
  crs = "+proj=stere +lat_0=-90 +lat_ts=-90 +lon_0=-63 +k=1 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
)
```

You might notice that some projections aren't "well-defined" on a global scale, and thus, may result in an error, or a "broken" looking map. For instance, [this Albers projection centered on Alaska](https://spatialreference.org/ref/epsg/3467/):

```{r alaska, fig.align = 'center', fig.height = 4, fig.width = 5}
alaska <- "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
plotdap("base", crs = alaska)
```

That does not mean we can't use this (or similar) projections -- we just have to be careful that they are sensible given the lat/lon limits. By default, those limits span the entire world, but as we'll see later, the limits are shrunk to the given data (i.e., `griddap()` / `tabledap()`) limits. In other words, we should expect this projection to work once we "add" some data located near Alaska to the visualization. However, in case you want to make a map without any data, or want to customize the background map in some special way, you can supply an [**sf**](https://CRAN.R-project.org/package=sf) object (or something coercable to an **sf** object) to the `mapData` argument.

```{r, echo = FALSE}
alaska <- "+proj=aea +lat_1=55 +lat_2=65 +lat_0=50 +lon_0=-154 +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
```

```{r usmap, fig.align = 'center', fig.height = 4, fig.width = 5}
library(sf)
library(mapdata)
w <- st_as_sf(maps::map("world", plot = FALSE, fill = TRUE))
us <- st_transform(subset(w, ID == "USA"), alaska)
plotdap(mapData = us)
```

With the odd exception of window sizing and projections, the options in `plotdap()` should *just work* in a similar way for either plotting method. However, there are some useful options that are deliberately left out, since they work differently based on the plotting method.

The `mapData` argument can be used to change the resolution of the continental outlines or to limit the outline to a pre-selected area, which can speed up processing because fewer unused polygons need to be clipped. This is particularly important for maps that will cross the dateline. So for example to use the hi-res outlines for VIIIRS SST off the coast of North America:

```{r get_viirsSST, eval = FALSE }
sstInfo <- rerddap::info('erdVHsstaWS3day', url = 'https://coastwatch.pfeg.noaa.gov/erddap/')
viirsSST <- rerddap::griddap(sstInfo, 
                             latitude = c(41., 31.), 
                             longitude = c(-128., -115), 
                             time = c('2026-03-18T12:00:00Z','2026-03-18T12:00:00Z'), 
                             fields = 'sst')

```

```{r viirs_hires,echo = TRUE, eval = FALSE}
w <- map("worldHires", xlim = c(-140., -114), ylim = c(30., 42.), 
         fill = TRUE, plot = FALSE)
# map using that outline,  temperature color from cmocean
add_griddap(plotdap(mapData = w), viirsSST, ~sst, fill = "thermal" )

```

![](viirs_hires.png)

Maps that cross the dateline work better using the "world2" or "world2Hires" databases frpm the `mapdata` package. There is a known problem with that continental database in that polygons from certain countries cause artificial lines in the map, and must removed, as done below.

```{r world2Hires, fig.align = 'center', fig.height = 3, fig.width = 5,  message = FALSE, warning = FALSE}
xpos <- c(135.25, 240.25)
ypos <- c(20.25, 60.25)
zpos <- c(70.02, 70.02)
remove <- c("UK:Great Britain", "France", "Spain", "Algeria", "Mali", "
            Burkina Faso", "Ghana", "Togo")
#subset world2Hires with those countries removed
w <- map("world2Hires", plot = FALSE, fill = TRUE, ylim = ypos, xlim = xpos)
w <- map("world2Hires", regions = w$names[!(w$names %in% remove)], 
         plot = FALSE, fill = TRUE, ylim = ypos, xlim = xpos)
# plot result
plotdap(mapData = w)

```

Since the result of `plotdap()` is always a map, it always forces a fixed aspect ratio (i.e., $\frac{height}{width}$ of graph equals $r=\frac{latitude}{longitude}$). For this reason, the current size of your graphics device may not be sensible for the value of $r$ (for instance, if $r$ is high, but the height of the graphics device is small, you may see an error such as: `polygon edge not found` since the device cannot possibly render the result under the conditions). For a number of reasons, `plotdap()` will not automatically resize your graphics device; instead, it's recommended that you use a reliable graphics device such as Cairo, and use a height/width ratio close to $r$.

```{r cairo, eval = FALSE,  echo = TRUE, warning = FALSE, message = FALSE}
# write plot to disk using the Cairo package
library(Cairo)
# (latitude limits) / (longitude limits)
r <- 85 / 120
CairoPNG("myPlot.png", height = 400 * r, width = 400, res = 96)
# alter default margins for base plotting 
# (leaving just enough space for a title)
par(mar = c(0, 0, 1, 0))
plotdap("base", mapData = us, mapTitle = "Albers projection of Alaska")
dev.off()
```

More advanced users that know some base/ggplot2 plotting may want more control of certain aspects of the plot (a later section -- Customizing `plotdap()` objects -- covers this topic).

## Adding `tabledap()` layers

The `add_tabledap()` function allows you to add markers that encode variable(s) obtained via `tabledap()` to an existing `plotdap()` object. For example, suppose we have the following `sardines` data, and wish to understand the frequency of subsample counts:

```{r get_sardines, echo = TRUE, eval = FALSE}
my_url <- 'https://coastwatch.pfeg.noaa.gov/erddap/'
sardines <- tabledap(
  'FRDCPSTrawlLHHaulCatch',
  fields = c('latitude',  'longitude', 'time', 'scientific_name', 
             'subsample_count'),
  'time>=2010-01-01', 'time<=2012-01-01', 'scientific_name="Sardinops sagax"',
   url = my_url)
```

At the very least, `add_tabledap()` needs a base map (i.e., a `plotdap()` object), the `tabledap()` data, and a *formula* defining the variable of interest (for encoding the color of the markers). In R, you can create a formula by prefixing `~` to some expression. This formula can simply reference a variable already residing in the dataset (e.g., `~subsample_count`) or it can be a function of some variables (e.g. `~log2(subsample_count)`):

```{r , echo = TRUE, eval = FALSE}
p1 <- add_tabledap(
  plotdap(crs = "+proj=robin",  mapTitle = "subsample count"), 
  sardines, 
  ~subsample_count
)
p2 <- add_tabledap(
  plotdap(crs = "+proj=robin", mapTitle = "Log subsample count"), 
  sardines, 
  ~log2(subsample_count)
) 

p1
p2

```

![](sardines.png)

## Modifying `tabledap()` layers

It is also easy to alter the color scale as well as the symbol type and size in `add_tabledap()` via the `color`, `shape` and `size` arguments.

```{r, echo = TRUE, eval = FALSE}
p1 <- add_tabledap(
  plotdap(crs = "+proj=robin", mapTitle = "Sardines - change color"), 
  sardines, 
  ~subsample_count, 
  color = "dense", 
)
p2  <- add_tabledap(
  plotdap(crs = "+proj=robin", mapTitle = "Sardines - change shape and size"), 
  sardines, 
  ~subsample_count, 
  shape = 4,
  size = 1.
)
p1
p2
```

![](sardines2.png)

For further details about these arguments, please refer to the documentation on `help(add_tabledap)`.

## Adding `griddap()` layers

Similar to `add_tabledap()`, the `add_griddap()` function makes it easy to add rasters (i.e., rectangular tiles) to a `plotdap()` object. To demonstrate, lets obtain some of the latest sea surface temperatures along the western coast of the US.

```{r get_mur, eval = FALSE}
murSST_west <- griddap(
  'jplMURSST41', 
  latitude = c(22, 51), 
  longitude = c(-140, -105),
  time = c('last', 'last'), 
  fields = 'analysed_sst'
  )
```

Again, similar to `add_tabledap()`, `add_griddap()` needs a base map (i.e., a `plotdap()` object), the `griddap()` data, and a *formula* defining the variable of interest (for encoding the fill of the rectangles). The `add_griddap()` function also has a `maxpixels` argument which sets a maximum threshold for the number of cells (i.e., pixels) to use before projection and plotting occurs. Compared to ggplot2, base plotting is much more efficient at rendering raster objects, so it might be worth increasing the threshold in that case:

```{r plot_mur, echo = TRUE, eval = FALSE}
add_griddap(
  plotdap(crs = "+proj=robin"), 
  murSST_west, 
  ~analysed_sst, 
  maxpixels = 50000
)
```

![](murSST_west.png)

The `murSST_west` grid has a single time point (i.e., `length(unique(murSST_west$data$time)) == 1`), but what do we do when there are multiple time points? In addition to animating multiple grids (a la `add_tabledap()`), you also have the option to summarize multiple grids into a single grid. To demonstrate, lets grab some wind speeds measured along the west coast of the US.

```{r get_wind, eval = FALSE}
wind <- griddap(
  'erdQMwindmday', 
  time = c('2016-04-16', '2016-06-16'),
  latitude = c(30, 50), 
  longitude = c(210, 240),
  fields = 'y_wind'
)
```

When faced with multiple time periods, and `animate = FALSE` (the default), the `time` argument is used to reduce multiple grids (i.e., raster bricks) to a single grid (i.e., a single raster layer). You can pass any R function to the `time` argument, but when `animate = FALSE`, you should take care to ensure the function returns a single value. The default uses the `mean(na.rm = TRUE)` function so that each cell represents the average (in this case amongst three time points), but we could easily set this to `var()` to get the variance for each cell:

```{r, echo = TRUE, eval = FALSE}
p1 <- add_griddap(
  plotdap(mapTitle = "Mean Meridional Wind"), 
  wind, 
  ~y_wind, 
  fill = "delta", 
  time = mean
) 
my_func <- function(x) var(x, na.rm = TRUE)
p2 <- add_griddap(
  plotdap(mapTitle = "Variance of Meridional Wind"), 
  wind, 
  ~y_wind, 
  fill = "delta", 
  time = my_func
) 
p1
p2

```

![](winds.png)

### Changing the layer order

By default, `add_griddap` plots the land first and then the grid on top of that. For many uses that is desirable, but at other times it is more desirable to have the land mask the grid. This can now be done in the print method, by saving the `plotdap` object and printing with the option "landmask = TRUE".

```{r viirsSST_gridland, echo = TRUE, eval = FALSE}
plotdap(mapTitle = "Grid over Land") %>%
      add_griddap(
        viirsSST, 
        ~sst, 
        fill = "thermal"
        )

```

![](viirsSST_gridland.png)

and compare when land is plotted over the grid:

```{r viirsSST_landgrid, echo = TRUE, eval = FALSE}
plotdap(mapTitle = "Land Over Grid") %>%
    add_griddap(
      viirsSST, 
      ~sst, 
      fill = "thermal"
      ) %>%
    print(landmask = TRUE)
```

![](viirsSST_landgrid.png)

### maxpixels

Images of satellite data can contain a large number of pixels. `add_griddap()` allows for a differing number of pixels to be used (default is 10,000) by setting the parameter "maxpixels". When the actual number pixels is larger than the value of "maxpixels", the image is sub-sampled. This can greatly affect the how the image looks.

```{r maxpix_10, echo = TRUE, eval = FALSE}
plotdap(mapTitle = "maxpixels = 10,000") %>%
    add_griddap(
      viirsSST, 
      ~sst, 
      fill = "thermal",
      maxpixels = 10000 
      )
```

![](viirsSST_10000.png)

```{r maxpix_50, echo = TRUE, eval = FALSE}
plotdap(mapTitle = "maxpixels = 50,000") %>%
     add_griddap(
       viirsSST, 
       ~sst, 
       fill = "thermal", 
       maxpixels = 50000
       )
```

![](viirsSST_50000.png)

```{r maxpix_100, echo = TRUE, eval = FALSE}
plotdap(mapTitle = "maxpixels = 100,000") %>%
    add_griddap(
      viirsSST, 
      ~sst, 
      fill = "thermal",  
      maxpixels = 100000
      )
```

![](viirsSST_100000.png)

The increased resolution can really make a difference when the land masks the grid:

```{r maxpix_100_mask, echo = TRUE, eval = FALSE}
plotdap(mapTitle = "maxpixels = 100,000, landmask") %>%
    add_griddap(
      viirsSST, 
      ~sst, 
      fill = "thermal",  
      maxpixels = 100000
      ) %>%
    print(landmask = TRUE)
```

![](viirsSST_100000_mask.png)

### Working with other continental outlines

My default, `plotdap()` uses the global "world" database from the `map` package, and then trims the polygons based on the limit of the map. Higher resolution coastlines or coastlines that cross the dateline require can be used in `plotdap()`by passing this information in the function call. Similarly, computations can be sped up by pre-selecting the area of the coastline database before calling `plotdap()`.

To map the VIIRS SST data from the previous section using a subsetted, high-resolution continental outline, first the subset is defined for the high-resolution database:

```{r hires}
w <- map("worldHires", xlim = c(-130., -114), ylim = c(30., 42.), 
         fill = TRUE, plot = FALSE)
```

and then plotted, with the landmask option:

```{r viirs_hires_mask, echo = TRUE, eval = FALSE}
plotdap(mapData = w) %>%
    add_griddap(
      viirsSST, 
      ~sst, 
      fill = 'thermal',  
      maxpixels = 50000
      ) %>%
    print(landmask = TRUE)
```

![](viirsSST_hires_mask.png)

The “Simple Ocean Data Assimilation (SODA)” model (see <https://soda.umd.edu>) reconstructs the historical physical history of the ocean since the beginning of the 20th century. Here is the temperature at 70m depth for December 2015 for the North Pacific Ocean, as an example of using `plotdap()` when the data cross the dateline:

First, get the SODA data:

```{r soda70_get, echo = TRUE, eval = FALSE}

soda70Info <- rerddap::info('erdSoda331oceanmday')
xpos <- c(135.25, 240.25)
ypos <- c(20.25, 60.25)
zpos <- c(70.02, 70.02)
tpos <- c('2010-12-15', '2010-12-15')
soda70 <- rerddap::griddap(soda70Info,  
                  longitude = xpos, 
                  latitude = ypos, 
                  time = tpos, 
                  depth = zpos, 
                  fields = 'temp' )

```

Unfortunately some recent changes in `ggplot2` make it so `plotdap`can no longer directly cross the dateline, but it can be tricked into doing so. Since the data cross the dateline, it is necessary to use the “world2Hires” continental outlines in the package mapdata which is Pacific Ocean centered. Unfortunatley there is a small problem where the outlines from certain countries wrap and mistakenly appear in plots, and those countries must be removed, as done below.

By using the function 'add_ggplot()' `plotdap` cna still produce the desired plot:

```{r soda70, echo= TRUE, eval = FALSE}
remove <- c("UK:Great Britain", "France", "Spain", "Algeria", "Mali", 
            "Burkina Faso", "Ghana", "Togo")
#subset world2Hires with those countries removed
w <- map("mapdata::world2Hires", plot = FALSE, fill = TRUE, 
         ylim = ypos, xlim = xpos)
w <- map("mapdata::world2Hires", regions = w$names[!(w$names %in% remove)], 
         plot = FALSE, fill = TRUE, ylim = ypos, xlim = xpos)
cols <- cmocean::cmocean('thermal')(256)
r <- terra::rast(soda70$data[, c("longitude", "latitude", "temp")], type = "xyz")
r_poly <- terra::as.polygons(r)
r_sf <- sf::st_as_sf(r_poly)
sf::st_crs(r_sf) <- 4326
p <- plotdap(mapData = w)
p <- add_ggplot(p, ggplot2::geom_sf(data = r_sf, ggplot2::aes(fill = temp), color = NA) )
p <- add_ggplot(p, ggplot2::scale_fill_gradientn(name ='temp', colors = cols) )
p

```

![](soda70.png) An advantage of still using `plotdap` over just using `ggplot2` directly is the result is still a `plotdap` object in case it is desried to use `plotdap` to combine several `rerddap` extracts.

## Combining tables/grids

The `plotdap()` interface is designed so that you can plot both tables and grids at once (although I don't recommend trying to plot more than one `griddap()` or `tabledap()` object at once). It's also designed to work well with the `%>%` operator from the **magrittr** package so that code can be expressed in a left-to-right (rather than inside-out) fashion:

```{r overlay, echo = TRUE, eval = FALSE}
p <-   add_griddap(plotdap(), 
    murSST_west, 
    ~analysed_sst,
    fill = "thermal"
    ) 
p1 <-   add_tabledap(p,
    sardines, 
    ~subsample_count
    )
print(p1, landmask = TRUE)
```

![](overlay.png)

## Modifying `ggplotdap` objects

By this point, you might have noticed some subtle differences in the defaults of `plotdap("ggplot2")` versus `plotdap("base")`. The default base version is intentionally left minimal as it is often much harder (or impossible) to modify/remove elements from a base graphic once it's drawn. However, this is fairly easy using the **ggplot2** approach (assuming you know a bit of **ggplot2**), thanks to the `add_ggplot()` function (think of it like the `+` operator in **ggplot2**).

```{r modify, fig.align = 'center', fig.height = 4, fig.width = 5}
library(ggplot2)
plotdap(crs = "+proj=robin") %>%
  add_tabledap(
    sardines, 
    ~subsample_count, 
    size = 1
    ) %>%
  add_ggplot(
    labs(
      subtitle = "Sardinops sagax samples",
      caption = "Sardines are yummy"
    ), 
    theme_minimal(),
    theme(axis.ticks = element_blank(), axis.text = element_blank())
  )
```

Similarly, VIIRS SST plot in the previous section, detail is lost due to high values observed in landlocked water. To modify that plot:

```{r modify2, echo = TRUE, eval = FALSE, timeit = TRUE}
temp_color <- rerddap::colors$temperature
plotdap(mapTitle = "Reset colorscale limits") %>%
    add_griddap(
      viirsSST, 
      ~sst, 
      fill = "thermal"
      ) %>%
    add_ggplot(
       scale_fill_gradientn(colours = temp_color, na.value = NA, limits = c(10, 20)),
        scale_colour_gradientn(colors = temp_color, na.value = NA, limits = c(10, 20)),
        guides(colour = 'none')
     ) %>%
     print(landmask = TRUE)
```

![](modify2.png)

## Animating `plotdap` objects

Since `time` is an important variable in many `tabledap()` and `griddap()` objects, both functions have an `animate` argument which will animate the maps over the `time` variable. The animations only work for the `ggplot2` option, and are based on the latest version of `gganimate`.

`gganimate` is a very powerful package, and many of the options available are designed to allow for "tweening", that is for smooth animation between frames. Unfortunately for complex maps with many features, that can lead to very large animations that can take hours to render. For this reason, all `plotdap` animations use `gganimate::transition_manual` so that there is one frame for each time period, and no "tweening" between frames. The animation can be controlled by saving the animation to an object, and then displaying the saved object using `gganimate::animate()` to control the display. The

Since `time` is an important variable in many `tabledap()` objects, `add_tabledap()` also has a `animate` argument which allows you to easily animate over the `time` variable. It is also easy to alter the color scale and symbol type via the `color` and `shape` arguments. For details about these arguments, please refer to the documentation on `help(add_tabledap)`

```{r, eval = FALSE, echo = TRUE}
add_tabledap(
  plotdap(crs = "+proj=robin"), 
  sardines, 
  ~subsample_count, 
  color = "dense", 
  shape = 4, 
  animate = TRUE
)
```

![sardine animation](sardines.gif)

If you just want to animate the raw grid values over time (instead of computing summary statistic(s) for each cell), you can set `time` to the `identity` function, then set `animate = TRUE`:

```{r, eval = FALSE, echo = TRUE }
add_griddap(
  plotdap(crs = "+proj=robin"), 
  wind, ~y_wind, 
  time = identity, 
  fill = 'delta',
  animate = TRUE
)
```

![y_wind animation](y_wind.gif) The frames per second and the total number of frames for an animated `plotdap` object `p` can be controlled calling `gganimate::animate(p$ggplot, ...)` with the desired other settings. Unfortunately, the map bounding box set in `p` may not be what is desired, because when `plotdap` prints an object it calculates a good bounding box for a map, but that is not stored in the object `p`. An improved version of the utility function `bbox_set()` is included which will set the desired bounding box.

```{r, eval = FALSE, echo = TRUE }
p <- add_griddap(
  plotdap(crs = "+proj=robin"), 
  wind, ~y_wind, 
  time = identity, 
  fill = 'delta',
  animate = TRUE
)
  ylim <- c(30, 50)
  xlim <- c(-150, -120)
  p <- bbox_set(p, xlim, ylim)
```

## Contour Plots {#contours}

The function `add_griddap_contours()` adds contours to an existing `plotdap::add_griddap()` object, using the R package 'isoband'. The function builds on an existing 'plotdap' object and also requires the dataframe that was used in creating the plot. The function `add_griddap_contours()` is defined as:

```{r add_contour, eval = FALSE}
add_griddap_contours <- function(p,
                                 data,
                                 var,
                                 n_breaks  = 10L,
                                 breaks    = NULL,
                                 lon_col   = "longitude",
                                 lat_col   = "latitude",
                                 color     = "black",
                                 linewidth = 0.3,
                                 alpha     = 1,
                                 crs       = 4326L)
```

`add_griddap_contours()` allows the setting of the number of contours, specified contour breaks if desired, and settings for the color and width of the contour lines.

Repeating the previous example for the MURv4.1 SST dataset:

```{r mur2, eval = FALSE}
myURL <- "https://coastwatch.pfeg.noaa.gov/erddap/"
info <- rerddap::info("jplMURSST41", url = myURL)
murSST  <- griddap(info,
                   latitude  = c(30, 50),
                   longitude = c(-140, -110),
                   time      = c("2020-06-15", "2020-06-15"),
                   fields    = "analysed_sst")

p <- plotdap() |>
      add_griddap(murSST, ~analysed_sst)

```

a contour plot with the default 10 equally spaced contours is created by:

```{r countour1, eval = FALSE}
p1 <- p |> add_griddap_contours(murSST$data, "analysed_sst") |> add_ggplot()
```

![Default SST contours](contour1.png)

<br><br><br>

and to have the landmask over the contours:

```{r contour2, eval = FALSE}
print(p1, landmask = TRUE)

```

![Default SST contours with landmask](contour2.png)

<br><br><br>

Explicit contour breaks can be set:

```{r contour3, eval = FALSE}
p |> add_griddap_contours(murSST$data, "analysed_sst",
                              breaks = c(10, 12, 14, 16, 18, 20)) |> 
                               add_ggplot() |> 
                               print(landmask = TRUE)
```

![Explicit SST contours with landmask](contour3.png)

<br><br><br>

## Interactive `plotdap` objects and `bbox_set()` {#interactive}

A new verion of the function `bbox_set()` is now included. It has the arguments:

```{r, echo = TRUE, eval = FALSE}

bbox_set <- function(plotobj, landmask = TRUE, xlim = NULL, ylim = NULL, interactive = FALSE)

```

If the values for "xlim" and "ylim" are not given, then the function will determine the limits and projection of the `plotdap` object, and set this in the `ggplot2` part of the the `plotdap` object. If "interactive" is set to "TRUE", then a `ggplot2` object is returned that works properly with the function `plotly::ggplotly()`.

```{r, echo = TRUE, eval = FALSE}
library(plotly)

myURL <- "https://coastwatch.pfeg.noaa.gov/erddap/"

sstInfo <- rerddap::info('erdVHsstaWS3day', url = myURL)
# get latest 3-day composite sst
viirsSST <- rerddap::griddap(sstInfo, 
                             latitude = c(41., 31.), 
                             longitude = c(-128., -115), 
                             time = c('2026-03-18T12:00:00Z','2026-03-18T12:00:00Z'), 
                             fields = 'sst')

w <- map("worldHires", xlim = c(-140., -114), ylim = c(30., 42.), fill = TRUE, plot = FALSE)
my_plot <- add_griddap(plotdap(mapData = w), viirsSST, ~sst, fill = "thermal" )

my_plot1 <- bbox_set(my_plot, xlim = c(-128., -115), ylim = c(31, 41.), interactive = TRUE)

# must have '"tooltip = "text"' to work properly.
plotly::ggplotly(my_plot1, tooltip = "text")

```

The resulting plot is an interactive plot that as you mouse over the data will display the latitude, longitude and value at any point. As the plot takes a long time to create and is quote large, both of which cause problems with CRAN, it is not displayed here.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_ggplot.R
\name{add_ggplot}
\alias{add_ggplot}
\title{Add ggplot2 elements to a plotdap object}
\usage{
add_ggplot(plot, ...)
}
\arguments{
\item{plot}{a plotdap object.}

\item{...}{arguments passed along to \code{geom_sf()}
(if \code{method='ggplot2'}, otherwise ignored).}
}
\value{
A plotdap object
}
\description{
\code{add_ggplot} allows for plotdap ggplot maps to be modified by
further ggplot2 settings
}
\examples{

p <- plotdap(
  crs = "+proj=laea +y_0=0 +lon_0=155 +lat_0=-90 +ellps=WGS84 +no_defs")
p <- add_ggplot(
 p,
 ggplot2::theme_bw()
)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sardines.R
\docType{data}
\name{sardines}
\alias{sardines}
\title{sardine Data}
\format{
An object of class \code{tabledap} (inherits from \code{data.frame}) with 56 rows and 5 columns.
}
\usage{
sardines
}
\description{
pre-Download of sardine data in `add_tabledap()` example so that example
can run within CRAN Time limits
}
\details{
obtained using the `rerddap` command
  sardines <- tabledap( 'FRDCPSTrawlLHHaulCatch',
     fields = c('latitude',  'longitude', 'time', '
         scientific_name', 'subsample_count'),
        'time>=2010-01-01', 'time<=2012-01-01',
        'scientific_name="Sardinops sagax"')
)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotdap-package.R
\docType{package}
\name{plotdap-package}
\alias{plotdap-package}
\title{plotdap: Easily Visualize Data from 'ERDDAP™' Servers via the 'rerddap' Package}
\description{
Easily visualize and animate 'tabledap' and 'griddap' objects obtained via the
'rerddap' package in a simple one-line command,  using either base graphics or
'ggplot2' graphics. 'plotdap' handles extracting and reshaping the data,  map
 projections and continental outlines.  Optionally the data can be animated
 through time using the 'gganmiate' package.#'
}
\details{
The package contains four main functions:
}
\section{Main Functions}{

\itemize{
\item \code{plotdap} - Visualize data returned from rerddap servers.
\item \code{add_griddap} - Add rerddap::griddap() data to a plotdap map
\item \code{add_tabledap} - Add rerddap::tabledap data to a plotdap map
\item \code{add_ggplot} - Add ggplot2 elements to a plotdap object
 }
#'
 @section Details:
Besides the terse help documents,  more  detail in using the
functions are given  in the included vignette "using_plotdap". The datasets used
in the vignette are included in the "data" directory.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rmendels/plotdap}
  \item Report bugs at \url{https://github.com/rmendels/plotdap/issues}
}

}
\author{
\strong{Maintainer}: Roy Mendelssohn \email{roy.mendelssohn@noaa.gov} [contributor]

Authors:
\itemize{
  \item Roy Mendelssohn \email{roy.mendelssohn@noaa.gov} [contributor]
  \item Carson Sievert
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_griddap_contours.R
\name{add_griddap_contours}
\alias{add_griddap_contours}
\title{Add contour lines to a plotdap object}
\usage{
add_griddap_contours(
  p,
  data,
  var,
  n_breaks = 10L,
  breaks = NULL,
  lon_col = "longitude",
  lat_col = "latitude",
  color = "black",
  linewidth = 0.3,
  alpha = 1,
  crs = 4326L
)
}
\arguments{
\item{p}{a plotdap object (result of \code{plotdap()} piped through
\code{add_griddap()}).  \strong{Not} the result of
\code{add_ggplot()}.}

\item{data}{data frame  typically \code{dat$data} from a
\code{griddap()} call.}

\item{var}{character; name of the variable column to contour.}

\item{n_breaks}{integer; number of evenly-spaced interior contour levels
(ignored when \code{breaks} is supplied). Default 10.}

\item{breaks}{numeric vector of explicit contour levels, or \code{NULL}.}

\item{lon_col}{name of longitude column (default \code{"longitude"}).}

\item{lat_col}{name of latitude column  (default \code{"latitude"}).}

\item{color}{contour line color (default \code{"black"}).}

\item{linewidth}{contour line width (default \code{0.3}).}

\item{alpha}{opacity 01 (default \code{1}).}

\item{crs}{integer EPSG code for the data (default \code{4326}).}
}
\value{
the plotdap object with contour lines added to its internal ggplot.
  Pipe the result to \code{add_ggplot()} or \code{print()} as usual.
}
\description{
Computes isolines from a griddap data frame and appends them as a
\code{geom_sf()} layer to the \strong{plotdap object's internal ggplot}.
Call this \emph{before} \code{add_ggplot()} or \code{print()}.
}
\examples{
\dontrun{
  library(rerddap)
  library(plotdap)

  myURL <- "https://coastwatch.pfeg.noaa.gov/erddap/"
  info <- rerddap::info("jplMURSST41", url = myURL)
  dat  <- griddap(info,
            latitude  = c(30, 50),
            longitude = c(-140, -110),
            time      = c("2020-06-15", "2020-06-15"),
            fields    = "analysed_sst")

  p <- plotdap() |>
         add_griddap(dat, ~analysed_sst)

  ## Default: 10 evenly-spaced contour levels, then render
  p |> add_griddap_contours(dat$data, "analysed_sst") |> add_ggplot()

  ## Explicit break values
  p |> add_griddap_contours(dat$data, "analysed_sst",
                             breaks = c(10, 12, 14, 16, 18, 20)) |> add_ggplot()

  ## More levels, thinner grey lines
  p |> add_griddap_contours(dat$data, "analysed_sst",
                             n_breaks = 20, color = "grey40", linewidth = 0.2) |>
       add_ggplot()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/murSST.R
\docType{data}
\name{murSST}
\alias{murSST}
\title{murSST Data}
\format{
An object of class \code{griddap_nc} (inherits from \code{nc}, \code{data.frame}) with 0 rows and 2 columns.
}
\usage{
murSST
}
\description{
pre-Download of murSST in `add_griddap()` example so that example
can run within CRAN Time limits
}
\details{
obtained using the `rerddap` command
murSST <- griddap(
'jplMURSST41', latitude = c(22, 51), longitude = c(-140, -105),
time = c('last', 'last'), fields = 'analysed_sst'
)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QMwind.R
\docType{data}
\name{QMwind}
\alias{QMwind}
\title{QMwind Data}
\format{
An object of class \code{griddap_nc} (inherits from \code{nc}, \code{data.frame}) with 0 rows and 2 columns.
}
\usage{
QMwind
}
\description{
pre-Download of QMwind in `add_griddap()` example so that example
can run within CRAN Time limits
}
\details{
obtained using the `rerddap` command
wind <- griddap('erdQMwindmday',
       time = c('2016-11-16', '2017-01-16'), latitude = c(30, 50),
       longitude = c(210, 240),
       fields = 'x_wind')
)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotdap.R
\name{print.ggplotdap}
\alias{print.ggplotdap}
\title{Print a ggplot plotdap object}
\usage{
\method{print}{ggplotdap}(x, ...)
}
\arguments{
\item{x}{a ggplotdap object}

\item{...}{currently unused}
}
\description{
Print a ggplot plotdap object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bbox_set.R
\name{bbox_set}
\alias{bbox_set}
\title{change bounding box in plotdap object}
\usage{
bbox_set(
  plotobj,
  landmask = TRUE,
  xlim = NULL,
  ylim = NULL,
  interactive = FALSE
)
}
\arguments{
\item{plotobj}{valid plotdap object}

\item{landmask}{optional: if true land is plotted over the data}

\item{xlim}{optional: new x-values of the bounding box otherwise taken from the plotdap object}

\item{ylim}{optional: new y-values of the bounding box otherwise taken from the plotdap object}

\item{interactive}{optional: if true  creates an interactive version of the graphic}
}
\value{
a ggplot object with reset limits. If interactive is true,  then returns a
           ggplot object that will work with the `plotly` package.
}
\description{
\code{bbox_set}changes the bounding box in an plotdap object.
Particularly needed if using gganimate::animate()
}
\examples{
p <- plotdap()
p <- add_tabledap(p, sardines, ~subsample_count)
xlim = c(-125, -115)
ylim <- c(30., 50.)
p <- bbox_set(p, xlim = xlim, ylim = ylim)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotdap.R
\name{plotdap}
\alias{plotdap}
\title{Visualize rerddap data}
\usage{
plotdap(
  method = c("ggplot2", "base"),
  mapData = maps::map("world", plot = FALSE, fill = TRUE),
  crs = NULL,
  datum = sf::st_crs(4326),
  mapTitle = NULL,
  mapFill = "gray80",
  mapColor = "gray90",
  ...
)
}
\arguments{
\item{method}{the plotting method. Currently ggplot2 and base plotting
are supported.}

\item{mapData}{an object coercable to an sf object via \code{st_as_sf()}.}

\item{crs}{a coordinate reference system: integer with the epsg code,
or character with proj4string.}

\item{datum}{crs that provides datum to use when generating graticules.
Set to \code{NULL} to hide the graticule.}

\item{mapTitle}{a title for the map.}

\item{mapFill}{fill used for the map.}

\item{mapColor}{color used to draw boundaries of the map.}

\item{...}{arguments passed along to \code{geom_sf()}
(if \code{method='ggplot2'}, otherwise ignored).}
}
\value{
A plotdap object
}
\description{
Visualize data returned from rerddap servers. Use \code{plotdap()} to initialize
a plot, specify the plotting method (specifically, 'base' or 'ggplot2'),
and set some global options/parameters. Then use \code{add_tabledap()}
and/or \code{add_griddap()} to add "layers" of actual data to be visualized.
}
\details{
The "ggplot2" method is slower than "base" (especially
for high-res grids/rasters), but is more flexible/extensible. Additional ggplot2
layers, as well as scale defaults, labels, theming, etc. may be modified via
the \code{add_ggplot()} function. See the mapping vignette for an introduction
and overview of rerddap's visualization methods --
\code{browseVignettes(package = "rerddap")}.
}
\examples{

# base plotting tends to be faster (especially for grids),
# but is less extensible plotdap("base")

 plotdap()
 plotdap("base")


}
\seealso{
\code{tabledap()}, \code{griddap()}
}
\author{
Carson Sievert
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_tabledap.R
\name{add_tabledap}
\alias{add_tabledap}
\title{Add rerddap::tabledap data to a plotdap map}
\usage{
add_tabledap(
  plot,
  table,
  var,
  color = c("#132B43", "#56B1F7"),
  size = 1.5,
  shape = 19,
  animate = FALSE,
  cumulative = FALSE,
  ...
)
}
\arguments{
\item{plot}{a \link[plotdap]{plotdap} object.}

\item{table}{a \link[rerddap]{tabledap} object.}

\item{var}{a formula defining a variable, or function of variables to visualize.}

\item{color}{either a character string of length 1 matching a name in \code{cmocean}
or a vector of color codes. This defines the colorscale used to encode values
of \code{var}.}

\item{size}{the size of the symbol.}

\item{shape}{the shape of the symbol. For valid options, see the 'pch' values
section on \link{points}. \code{plot(0:25, 0:25, pch = 0:25)} also gives a
quick visual of the majority of possibilities.}

\item{animate}{whether to animate over the \code{time} variable (if it exists).
Currently only implemented for \code{method='ggplot2'} and requires the
gganimate package.}

\item{cumulative}{- if animation should be cumulative -default FALSE}

\item{...}{arguments passed along to \code{geom_sf()} (if \code{method='ggplot2'}, otherwise ignored).}
}
\value{
A plotdap object
}
\description{
\code{add_tabledap} adds the data from an 'rerddap::tabledap()' call to
a 'plotdap' map
}
\examples{

# base plotting tends to be faster,
# but is less extensible plotdap("base")

# test datasets in data folder to meet execution timings
# code given to extract the data

\donttest{
sardines <- rerddap::tabledap(
 'FRDCPSTrawlLHHaulCatch',
 fields = c('latitude',  'longitude', 'time', 'scientific_name', 'subsample_count'),
  'time>=2010-01-01', 'time<=2012-01-01',
  'scientific_name="Sardinops sagax"'
  )
}

p <- plotdap()
p1 <- add_tabledap(p, sardines, ~subsample_count)
p2 <- add_tabledap(p, sardines, ~log2(subsample_count))

# using base R plotting
p <- plotdap("base")
p <- add_tabledap(p, sardines, ~subsample_count)

# robinson projection
p <- plotdap(crs = "+proj=robin")
p <- add_tabledap(p, sardines, ~subsample_count)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_griddap.R
\name{add_griddap}
\alias{add_griddap}
\title{Add rerddap::griddap() data to a plotdap map}
\usage{
add_griddap(
  plot,
  grid,
  var,
  fill = "viridis",
  maxpixels = 10000,
  time = mean,
  animate = FALSE,
  cumulative = FALSE,
  ...
)
}
\arguments{
\item{plot}{a \link{plotdap} object.}

\item{grid}{a \link[rerddap]{griddap} object.}

\item{var}{a formula defining a variable, or function of variables to visualize.}

\item{fill}{either a character string of length 1 matching a name in the
package \code{cmocean} or a vector of color codes.
This defines the colorscale used to encode values
of \code{var}.}

\item{maxpixels}{integer > 0. Maximum number of cells to use for the plot.
If maxpixels < ncell(x), sampleRegular is used before plotting.
If gridded=TRUE maxpixels may be ignored to get a larger sample}

\item{time}{how to resolve multiple time frames. Choose one of the following:
\itemize{
  \item A function to apply to each observation at a particular location
  (\link{mean} is the default).
  \item A character string (of length 1) matching a time value.
}}

\item{animate}{whether to animate over the \code{time} variable (if it exists).
Currently only implemented for \code{method='ggplot2'} and requires the
gganimate package.}

\item{cumulative}{- if animation should be cumulative -default FALSE}

\item{...}{arguments passed along to \code{geom_sf()}
(if \code{method='ggplot2'}, otherwise ignored).}
}
\value{
A plotdap object
}
\description{
\code{add_griddap} adds the data from an 'rerddap::griddap() call to
a 'plotdap' map
}
\examples{

# base plotting tends to be faster,
# but is less extensible plotdap("base")

# actual datasets in data folder to meet execution timings


 # murSST <- rerddap::griddap(
 #  ' jplMURSST41', latitude = c(35, 40), longitude = c(-125, -120.5),
 #   time = c('last', 'last'), fields = 'analysed_sst'
 # )

 # QMwind <- rerddap::griddap(
 #  'erdQMwindmday', time = c('2016-11-16', '2017-01-16'),
 #  latitude = c(30, 50), longitude = c(210, 240),
 #  fields = 'x_wind'
 #  )

p <- plotdap(crs = "+proj=robin")
p <- add_griddap(p, murSST, ~analysed_sst)

 # p <- plotdap(mapTitle = "Average wind over time")
 # p <- add_griddap(p, QMwind, ~x_wind)

# p <- plotdap("base", crs = "+proj=robin")
# p <- add_griddap(p, murSST, ~analysed_sst)

# layer tables on top of grids
require(magrittr)
p <- plotdap("base") \%>\%
  add_griddap(murSST, ~analysed_sst) \%>\%
  add_tabledap(sardines, ~subsample_count)

# multiple time periods
p <- plotdap("base", mapTitle = "Average wind over time")
p <- add_griddap(p, QMwind, ~x_wind)

}
